; $Id: fisv_eventcb.pro,v 1.42 2007/04/24 08:54:42 makiuti Exp $
;
; FIS Data Visualizer (FISv) IDL Event Callback Procedures
;
; Copyright (c) 2003-2007, AKARI(ASTRO-F) Data Reduction Team. All rights reserved.
;
;+
; NAME:
;       FISV_EVENTCB
;
; PURPOSE:
;       The purpose of this program is to display a time series data (TSD)
;       of the ASTRO-F/FIS survey data.
;
; AUTHOR:
;       ASTRO-F Data Reduction team
;       Web Page: http://www.ir.isas.jaxa.jp/AKARI/iris_data/
;
; CATEGORY:
;       FIS Data Visualizer
;       Widgets program
;
; CALLING SEQUENCE:
;       fisv_eventcb.pro is called from fisv.pro
;
; INPUTS:
;       None.
;
; OUTPUTS:
;       None.
;
; RESTRICTIONS:
;
;
; PROCEDURE:
;       First, open a TSD file (fits binary table format).
;
; EXAMPLE:
;
;       To start FIS Data Visualizer (FISv), type:
;       IDL> fisv
;
; OTHER PROGRAMS NEEDED:
;
;   The following programs are required to run this one:
;
;    xydata.pro
;    sw_tree_wid.pro
;    lw_tree_wid.pro
;    text_dump.pro
;    save_vdet.pro
;    list_vdet.pro
;    list_vdet_ps.pro
;    view_fits_header.pro
;    make_initial_tree.pro
;    make_x_status_tree.pro
;    make_x_anlgtele_tree.pro
;    make_x_vdet_tree.pro
;    make_x_flux_tree.pro
;    make_x_ferr_tree.pro
;    make_x_pixflag_tree.prp
;    make_x_pixflag_ch_tree.pro
;    make_x_quality_tree.pro
;    make_x_quality_ch_tree.pro
;    make_x_counter_tree.pro
;    make_x_counter_ch_tree.pro
;    make_x_fis_hk_tree.pro
;    make_x_irc_hk_tree.pro
;    make_x_hk_2_tree.pro
;    make_x_aocu_tree.pro
;    make_x_gads_tree.pro
;    make_x_pr_tree.pro
;    make_x_se_tree.pro
;    make_y_status_tree.pro
;    make_y_anlgtele_tree.pro
;    make_y_vdet_tree.pro
;    make_y_flux_tree.pro
;    make_y_ferr_tree.pro
;    make_y_pixflag_tree.prp
;    make_y_pixflag_ch_tree.pro
;    make_y_quality_tree.pro
;    make_y_quality_ch_tree.pro
;    make_y_counter_tree.pro
;    make_y_counter_ch_tree.pro
;    make_y_fis_hk_tree.pro
;    make_y_irc_hk_tree.pro
;    make_y_hk_2_tree.pro
;    make_y_aocu_tree.pro
;    make_y_gads_tree.pro
;    make_y_pr_tree.pro
;    make_y_se_tree.pro
;    tsdeditor.pro
;    maskflag.pro
;    aftime2date.pro
;    lds_query.pro
;    set_env_fisv.pro
;    mk_sub_tsd.pro
;    fis_temp.pro
;    gb_batch.pro
;
;    zplot_alt.pro
;    zplot_flag.pro
;      getcolor.pro(lib)
;    stat_plot.pro
;    fsc_psconfig__define.pro(lib)
;      fsc_droplist.pro(lib)
;      fsc_fileselect.pro(lib)
;      fsc_field.pro(lib)
;      fsc_plotwindow.pro(lib)
;      error_message.pro(lib)
;    det_image.pro
;      colorbar.pro(lib)
;
;    ..and other reduction routines (TSD, GB, etc.)
;
; REFERENCE:
;
;
; MODIFICATION HISTORY:
;        Written by: Makiuti,S.
;        20 Feb. 2003 Ver.0.2
;        28 Nov. 2003 Ver.0.70 (for TSD ver.1)
;        29 Nov. 2003 Ver.1.00 (for TSD ver.2)
;        08 Oct. 2004 Ver.2.00 (for TSD ver.3)
;        16 May. 2006 Ver.3.00 (for TSD ver.4)
;        30 Jan. 2007 Last modified
;-

;-----------------------------------------------------------------
; TSD file open
;-----------------------------------------------------------------
pro FileOpen, Event
COMMON Share, tsd

  ;# Initialize environment for FISv
  !p.multi = 0

  ; TSD file (FITS Binary) Open
  sFile = DIALOG_PICKFILE(FILTER=['*.fits;*.fits.gz'], /READ, DIALOG_PARENT = Event.top)

  ; Get widget IDs
  WID_BASE_2 = widget_info(Event.top, find_by_uname='WID_BASE_2')
  WID_BASE_3 = widget_info(Event.top, find_by_uname='WID_BASE_3')
  WID_BASE_10 = widget_info(Event.top, find_by_uname='WID_BASE_10')

  W_MENU_Plot = widget_info(Event.top, find_by_uname='W_MENU_Plot')
  W_MENU_Table = widget_info(Event.top, find_by_uname='W_MENU_Table')
  W_MENU_Print = widget_info(Event.top, find_by_uname='W_MENU_Print')
  W_MENU_PS = widget_info(Event.top, find_by_uname='W_MENU_PS')

  W_MENU_HeadView = widget_info(Event.top, find_by_uname='W_MENU_HeadView')
  W_MENU_MaskPlot = widget_info(Event.top, find_by_uname='W_MENU_MaskPlot')
  W_MENU_SetBad = widget_info(Event.top, find_by_uname='W_MENU_SetBad')
  W_MENU_DetImage = widget_info(Event.top, find_by_uname='W_MENU_DetImage')
  W_MENU_ShVlist = widget_info(Event.top, find_by_uname='W_MENU_ShVlist')
  W_MENU_PSVlist = widget_info(Event.top, find_by_uname='W_MENU_PSVlist')
  W_MENU_AfTime2Date = widget_info(Event.top, find_by_uname='W_MENU_AfTime2Date')
  W_MENU_SvVdet = widget_info(Event.top, find_by_uname='W_MENU_SvVdet')
  W_MENU_MkSubTSD = widget_info(Event.top, find_by_uname='W_MENU_MkSubTSD')
  W_MENU_FFT = widget_info(Event.top, find_by_uname='W_MENU_FFT')
  W_MENU_EDIT = widget_info(Event.top, find_by_uname='W_MENU_EDIT')
  W_MENU_ClearFlag = widget_info(Event.top, find_by_uname='W_MENU_ClearFlag')

  W_MENU_GBbatch = widget_info(Event.top, find_by_uname='W_MENU_GBbatch')
  W_MENU_FlgB = widget_info(Event.top, find_by_uname='W_MENU_FlgB')
  W_MENU_RAnm = widget_info(Event.top, find_by_uname='W_MENU_RAnm')
  W_MENU_CnvV = widget_info(Event.top, find_by_uname='W_MENU_CnvV')
  W_MENU_RCcr = widget_info(Event.top, find_by_uname='W_MENU_RCcr')
  W_MENU_Diff = widget_info(Event.top, find_by_uname='W_MENU_Diff')
  W_MENU_Peri = widget_info(Event.top, find_by_uname='W_MENU_Peri')
  W_MENU_MTGL = widget_info(Event.top, find_by_uname='W_MENU_MTGL')
  W_MENU_GPGL = widget_info(Event.top, find_by_uname='W_MENU_GPGL')
  W_MENU_MCLT = widget_info(Event.top, find_by_uname='W_MENU_MCLT')
  W_MENU_MDKT = widget_info(Event.top, find_by_uname='W_MENU_MDKT')
  W_MENU_DKSb = widget_info(Event.top, find_by_uname='W_MENU_DKSb')
  W_MENU_ResC = widget_info(Event.top, find_by_uname='W_MENU_ResC')
  W_MENU_FlxC = widget_info(Event.top, find_by_uname='W_MENU_FlxC')

  ; Set widgets sensitive OFF
  widget_control, WID_BASE_2, SENSITIVE=0
  widget_control, WID_BASE_3, SENSITIVE=0
  widget_control, WID_BASE_10, SENSITIVE=0
  widget_control, W_MENU_Plot, SENSITIVE=0
  widget_control, W_MENU_Table, SENSITIVE=0
  widget_control, W_MENU_Print, SENSITIVE=0
  widget_control, W_MENU_PS, SENSITIVE=0
  widget_control, W_MENU_HeadView, SENSITIVE=0
  widget_control, W_MENU_MaskPlot, SENSITIVE=0
  widget_control, W_MENU_SetBad, SENSITIVE=0
  widget_control, W_MENU_DetImage, SENSITIVE=0
  widget_control, W_MENU_ShVlist, SENSITIVE=0
  widget_control, W_MENU_PSVlist, SENSITIVE=0
  widget_control, W_MENU_AfTime2Date, SENSITIVE=0
  widget_control, W_MENU_SvVdet, SENSITIVE=0
  widget_control, W_MENU_MkSubTSD, SENSITIVE=0
  widget_control, W_MENU_FFT, SENSITIVE=0
  widget_control, W_MENU_EDIT, SENSITIVE=0
  widget_control, W_MENU_ClearFlag, SENSITIVE=0
  widget_control, W_MENU_GBbatch, SENSITIVE=0
  widget_control, W_MENU_FlgB, SENSITIVE=0
  widget_control, W_MENU_RAnm, SENSITIVE=0
  widget_control, W_MENU_CnvV, SENSITIVE=0
  widget_control, W_MENU_RCcr, SENSITIVE=0
  widget_control, W_MENU_Diff, SENSITIVE=0
  widget_control, W_MENU_Peri, SENSITIVE=0
  widget_control, W_MENU_MTGL, SENSITIVE=0
  widget_control, W_MENU_GPGL, SENSITIVE=0
  widget_control, W_MENU_MCLT, SENSITIVE=0
  widget_control, W_MENU_MDKT, SENSITIVE=0
  widget_control, W_MENU_DKSb, SENSITIVE=0
  widget_control, W_MENU_ResC, SENSITIVE=0
  widget_control, W_MENU_FlxC, SENSITIVE=0

  ; If any file was not selected
  IF (sFile EQ "") THEN BEGIN
    ; IF a file has been already opened
    IF OBJ_VALID(tsd) EQ 1 THEN BEGIN
      ; Set widgets sensitive ON again
      widget_control, WID_BASE_2, /SENSITIVE
      widget_control, WID_BASE_3, /SENSITIVE
      widget_control, WID_BASE_10, /SENSITIVE
      widget_control, W_MENU_Plot, /SENSITIVE
      widget_control, W_MENU_Table, /SENSITIVE
      widget_control, W_MENU_Print, /SENSITIVE
      widget_control, W_MENU_PS, /SENSITIVE
      widget_control, W_MENU_HeadView, /SENSITIVE
      widget_control, W_MENU_MaskPlot, /SENSITIVE
      widget_control, W_MENU_SetBad, /SENSITIVE
      widget_control, W_MENU_DetImage, /SENSITIVE
      widget_control, W_MENU_ShVlist, /SENSITIVE
      widget_control, W_MENU_PSVlist, /SENSITIVE
      widget_control, W_MENU_AfTime2Date, /SENSITIVE
      widget_control, W_MENU_SvVdet, /SENSITIVE
      widget_control, W_MENU_MkSubTSD, /SENSITIVE
      widget_control, W_MENU_FFT, /SENSITIVE
      widget_control, W_MENU_EDIT, /SENSITIVE
      widget_control, W_MENU_ClearFlag, /SENSITIVE
      widget_control, W_MENU_GBbatch, /SENSITIVE
      widget_control, W_MENU_FlgB, /SENSITIVE
      widget_control, W_MENU_RAnm, /SENSITIVE
      widget_control, W_MENU_CnvV, /SENSITIVE
      widget_control, W_MENU_RCcr, /SENSITIVE
      widget_control, W_MENU_Diff, /SENSITIVE
      widget_control, W_MENU_Peri, /SENSITIVE
      widget_control, W_MENU_MTGL, /SENSITIVE
      widget_control, W_MENU_GPGL, /SENSITIVE
      widget_control, W_MENU_MCLT, /SENSITIVE
      widget_control, W_MENU_MDKT, /SENSITIVE
      widget_control, W_MENU_DKSb, /SENSITIVE
      widget_control, W_MENU_ResC, /SENSITIVE
      widget_control, W_MENU_FlxC, /SENSITIVE
    ENDIF
    Return
  ENDIF

  ; Get ID of the LABEL Widget showing the open file
  WID_LABEL_0 = widget_info(Event.top, find_by_uname='WID_LABEL_0')
  ; Set value to LABEL Widget
  widget_control, WID_LABEL_0, set_value=sFile

  ; Get detector type (SW or LW) of the previous TSD file
  IF (OBJ_VALID(tsd) EQ 1) THEN BEGIN
    if (tsd->Get_TSDVER() ge 3) then begin
      pre_detector = (STRMATCH(tsd->Get_Datatype(), '*SW*', /FOLD_CASE) EQ 1) ? 0:1
    endif else begin
      pre_detector = -1 ; neither SW nor LW
    endelse
  ENDIF ELSE pre_detector = -1 ; neither SW nor LW


  WIDGET_CONTROL, /HOURGLASS
  ; First, destroy all existing pointer heap variables
  ;OBJ_DESTROY, OBJ_VALID()
  IF (OBJ_VALID(tsd)) THEN OBJ_DESTROY, tsd
  ; Make TSD object
  tsd=Obj_New('tsd')
  ret=tsd->Read_TSD(sFile)

  ; In the case that file name mismatch.
  IF (ret EQ -1) THEN BEGIN
    Result = DIALOG_MESSAGE('File Open Failed.', $
        TITLE='File Open result', /INFORMATION, DIALOG_PARENT=Event.top)
    IF (OBJ_VALID(tsd)) THEN OBJ_DESTROY, tsd
    Return
  ENDIF

  ; TSD version check
  IF (tsd->Get_TSDVER() NE 4) THEN BEGIN
    r=Dialog_message(['This TSD file is not Ver.4.', 'Use FISv of older versions for TSD Ver.1,2&3.'], $
        /INFORMATION, DIALOG_PARENT=Event.top)
    ;OBJ_DESTROY, OBJ_VALID()
    IF (OBJ_VALID(tsd)) THEN OBJ_DESTROY, tsd
    Return
  ENDIF

  ;## Remake Two Data Selector Trees ##
  IF (STRMATCH(tsd->Get_Datatype(), '*SW*', /FOLD_CASE) EQ 1) THEN BEGIN
      sw_tree_wid, Event, pre_detector
  ENDIF ELSE IF (STRMATCH(tsd->Get_Datatype(), '*LW*', /FOLD_CASE) EQ 1) THEN BEGIN
      lw_tree_wid, Event, pre_detector
  ENDIF ELSE BEGIN
      print, 'This is not FIS data'
      Return
  ENDELSE

  ; Get ID of Info Box
  WID_TEXT_0 = widget_info(Event.top, find_by_uname='WID_TEXT_0')
  ; Show information
  WIDGET_CONTROL, WID_TEXT_0, get_value = textvalue
  WIDGET_CONTROL, WID_TEXT_0, set_value = sFile+' has been opened.', $
    /append, SET_TEXT_TOP_LINE=n_elements(textvalue)-2

  ; Set widgets sensitive ON
  widget_control, WID_BASE_2, /SENSITIVE
  widget_control, WID_BASE_3, /SENSITIVE
  widget_control, WID_BASE_10, /SENSITIVE
  widget_control, W_MENU_Plot, /SENSITIVE
  widget_control, W_MENU_Table, /SENSITIVE
  widget_control, W_MENU_Print, /SENSITIVE
  widget_control, W_MENU_PS, /SENSITIVE
  widget_control, W_MENU_HeadView, /SENSITIVE
  widget_control, W_MENU_MaskPlot, /SENSITIVE
  widget_control, W_MENU_SetBad, /SENSITIVE
  widget_control, W_MENU_DetImage, /SENSITIVE
  widget_control, W_MENU_ShVlist, /SENSITIVE
  widget_control, W_MENU_PSVlist, /SENSITIVE
  widget_control, W_MENU_AfTime2Date, /SENSITIVE
  widget_control, W_MENU_SvVdet, /SENSITIVE
  widget_control, W_MENU_MkSubTSD, /SENSITIVE
  widget_control, W_MENU_FFT, /SENSITIVE
  widget_control, W_MENU_EDIT, /SENSITIVE
  widget_control, W_MENU_ClearFlag, /SENSITIVE
  widget_control, W_MENU_GBbatch, /SENSITIVE
  widget_control, W_MENU_FlgB, /SENSITIVE
  widget_control, W_MENU_RAnm, /SENSITIVE
  widget_control, W_MENU_CnvV, /SENSITIVE
  widget_control, W_MENU_RCcr, /SENSITIVE
  widget_control, W_MENU_Diff, /SENSITIVE
  widget_control, W_MENU_Peri, /SENSITIVE
  widget_control, W_MENU_MTGL, /SENSITIVE
  widget_control, W_MENU_GPGL, /SENSITIVE
  widget_control, W_MENU_MCLT, /SENSITIVE
  widget_control, W_MENU_MDKT, /SENSITIVE
  widget_control, W_MENU_DKSb, /SENSITIVE
  widget_control, W_MENU_ResC, /SENSITIVE
  widget_control, W_MENU_FlxC, /SENSITIVE

end
;-----------------------------------------------------------------
; TSD file open from LDS
;-----------------------------------------------------------------
pro FileOpenLDS, Event
COMMON Share, tsd

  ;# Initialize environment for FISv
  !p.multi = 0

  ; Get widget IDs
  WID_BASE_2 = widget_info(Event.top, find_by_uname='WID_BASE_2')
  WID_BASE_3 = widget_info(Event.top, find_by_uname='WID_BASE_3')
  WID_BASE_10 = widget_info(Event.top, find_by_uname='WID_BASE_10')

  W_MENU_Plot = widget_info(Event.top, find_by_uname='W_MENU_Plot')
  W_MENU_Table = widget_info(Event.top, find_by_uname='W_MENU_Table')
  W_MENU_Print = widget_info(Event.top, find_by_uname='W_MENU_Print')
  W_MENU_PS = widget_info(Event.top, find_by_uname='W_MENU_PS')

  W_MENU_HeadView = widget_info(Event.top, find_by_uname='W_MENU_HeadView')
  W_MENU_MaskPlot = widget_info(Event.top, find_by_uname='W_MENU_MaskPlot')
  W_MENU_SetBad = widget_info(Event.top, find_by_uname='W_MENU_SetBad')
  W_MENU_DetImage = widget_info(Event.top, find_by_uname='W_MENU_DetImage')
  W_MENU_ShVlist = widget_info(Event.top, find_by_uname='W_MENU_ShVlist')
  W_MENU_PSVlist = widget_info(Event.top, find_by_uname='W_MENU_PSVlist')
  W_MENU_AfTime2Date = widget_info(Event.top, find_by_uname='W_MENU_AfTime2Date')
  W_MENU_SvVdet = widget_info(Event.top, find_by_uname='W_MENU_SvVdet')
  W_MENU_MkSubTSD = widget_info(Event.top, find_by_uname='W_MENU_MkSubTSD')
  W_MENU_FFT = widget_info(Event.top, find_by_uname='W_MENU_FFT')
  W_MENU_EDIT = widget_info(Event.top, find_by_uname='W_MENU_EDIT')
  W_MENU_ClearFlag = widget_info(Event.top, find_by_uname='W_MENU_ClearFlag')

  W_MENU_GBbatch = widget_info(Event.top, find_by_uname='W_MENU_GBbatch')
  W_MENU_FlgB = widget_info(Event.top, find_by_uname='W_MENU_FlgB')
  W_MENU_RAnm = widget_info(Event.top, find_by_uname='W_MENU_RAnm')
  W_MENU_CnvV = widget_info(Event.top, find_by_uname='W_MENU_CnvV')
  W_MENU_RCcr = widget_info(Event.top, find_by_uname='W_MENU_RCcr')
  W_MENU_Diff = widget_info(Event.top, find_by_uname='W_MENU_Diff')
  W_MENU_Peri = widget_info(Event.top, find_by_uname='W_MENU_Peri')
  W_MENU_MTGL = widget_info(Event.top, find_by_uname='W_MENU_MTGL')
  W_MENU_GPGL = widget_info(Event.top, find_by_uname='W_MENU_GPGL')
  W_MENU_MCLT = widget_info(Event.top, find_by_uname='W_MENU_MCLT')
  W_MENU_MDKT = widget_info(Event.top, find_by_uname='W_MENU_MDKT')
  W_MENU_DKSb = widget_info(Event.top, find_by_uname='W_MENU_DKSb')
  W_MENU_ResC = widget_info(Event.top, find_by_uname='W_MENU_ResC')
  W_MENU_FlxC = widget_info(Event.top, find_by_uname='W_MENU_FlxC')


  ; Set widgets sensitive OFF
  widget_control, WID_BASE_2, SENSITIVE=0
  widget_control, WID_BASE_3, SENSITIVE=0
  widget_control, WID_BASE_10, SENSITIVE=0
  widget_control, W_MENU_Plot, SENSITIVE=0
  widget_control, W_MENU_Table, SENSITIVE=0
  widget_control, W_MENU_Print, SENSITIVE=0
  widget_control, W_MENU_PS, SENSITIVE=0
  widget_control, W_MENU_HeadView, SENSITIVE=0
  widget_control, W_MENU_MaskPlot, SENSITIVE=0
  widget_control, W_MENU_SetBad, SENSITIVE=0
  widget_control, W_MENU_DetImage, SENSITIVE=0
  widget_control, W_MENU_ShVlist, SENSITIVE=0
  widget_control, W_MENU_PSVlist, SENSITIVE=0
  widget_control, W_MENU_AfTime2Date, SENSITIVE=0
  widget_control, W_MENU_SvVdet, SENSITIVE=0
  widget_control, W_MENU_MkSubTSD, SENSITIVE=0
  widget_control, W_MENU_FFT, SENSITIVE=0
  widget_control, W_MENU_EDIT, SENSITIVE=0
  widget_control, W_MENU_ClearFlag, SENSITIVE=0
  widget_control, W_MENU_GBbatch, SENSITIVE=0
  widget_control, W_MENU_FlgB, SENSITIVE=0
  widget_control, W_MENU_RAnm, SENSITIVE=0
  widget_control, W_MENU_CnvV, SENSITIVE=0
  widget_control, W_MENU_RCcr, SENSITIVE=0
  widget_control, W_MENU_Diff, SENSITIVE=0
  widget_control, W_MENU_Peri, SENSITIVE=0
  widget_control, W_MENU_MTGL, SENSITIVE=0
  widget_control, W_MENU_GPGL, SENSITIVE=0
  widget_control, W_MENU_MCLT, SENSITIVE=0
  widget_control, W_MENU_MDKT, SENSITIVE=0
  widget_control, W_MENU_DKSb, SENSITIVE=0
  widget_control, W_MENU_ResC, SENSITIVE=0
  widget_control, W_MENU_FlxC, SENSITIVE=0

  ; Get detector type (SW or LW) of the previous TSD file
  IF (OBJ_VALID(tsd) EQ 1) THEN BEGIN
    if (tsd->Get_TSDVER() ge 3) then begin
      pre_detector = (STRMATCH(tsd->Get_Datatype(), '*SW*', /FOLD_CASE) EQ 1) ? 0:1
    endif else begin
      pre_detector = -1 ; neither SW nor LW
    endelse
  ENDIF ELSE pre_detector = -1 ; neither SW nor LW

  ;### Query Local Dala Server and make TSD object ###
  lds_query, GROUP_LEADER=Event.top

  ;### Error handling ###
  IF OBJ_VALID(tsd) NE 1 THEN Return

  ; If the current file name cannot be taken
  tsd_file_name = tsd->get_filename()
  IF strlen(tsd_file_name) EQ 0 THEN BEGIN
    r=Dialog_message(['LDS access has failed.'], $
        /INFORMATION, DIALOG_PARENT=Event.top)
    IF (OBJ_VALID(tsd)) THEN OBJ_DESTROY, tsd
    Return
  ENDIF

  ; Get ID of the LABEL Widget showing the current file name
  WID_LABEL_0 = widget_info(Event.top, find_by_uname='WID_LABEL_0')
  ; Set value to LABEL Widget
  widget_control, WID_LABEL_0, set_value = tsd_file_name

  ; TSD version check
  IF (tsd->Get_TSDVER() NE 4) THEN BEGIN
    r=Dialog_message(['This TSD file is not Ver.4.', 'Use FISv of older versions for TSD Ver.1,2&3.'], $
        /INFORMATION, DIALOG_PARENT=Event.top)
    ;OBJ_DESTROY, OBJ_VALID()
    IF (OBJ_VALID(tsd)) THEN OBJ_DESTROY, tsd
    Return
  ENDIF

  ;## Remake Two Data Selector Trees ##
  IF (STRMATCH(tsd->Get_Datatype(), '*SW*', /FOLD_CASE) EQ 1) THEN BEGIN
      sw_tree_wid, Event, pre_detector
  ENDIF ELSE IF (STRMATCH(tsd->Get_Datatype(), '*LW*', /FOLD_CASE) EQ 1) THEN BEGIN
      lw_tree_wid, Event, pre_detector
  ENDIF ELSE BEGIN
      print, 'This is not FIS data'
      Return
  ENDELSE

  ; Get ID of Info Box
  WID_TEXT_0 = widget_info(Event.top, find_by_uname='WID_TEXT_0')
  ; Show information
  WIDGET_CONTROL, WID_TEXT_0, get_value = textvalue
  IF (STRMATCH(tsd->Get_Datatype(), '*SW*', /FOLD_CASE) EQ 1) THEN BEGIN
      str='SW data have been gotten from the LDS.'
  ENDIF ELSE BEGIN str='LW data has been gotten from the LDS'
  ENDELSE
  WIDGET_CONTROL, WID_TEXT_0, set_value = str, /append, SET_TEXT_TOP_LINE=n_elements(textvalue)-2
  WIDGET_CONTROL, WID_TEXT_0, set_value = tsd->get_filename()+' has been opened.', $
    /append, SET_TEXT_TOP_LINE=n_elements(textvalue)-2

  ; Set widgets sensitive ON
  widget_control, WID_BASE_2, /SENSITIVE
  widget_control, WID_BASE_3, /SENSITIVE
  widget_control, WID_BASE_10, /SENSITIVE
  widget_control, W_MENU_Plot, /SENSITIVE
  widget_control, W_MENU_Table, /SENSITIVE
  widget_control, W_MENU_Print, /SENSITIVE
  widget_control, W_MENU_PS, /SENSITIVE
  widget_control, W_MENU_HeadView, /SENSITIVE
  widget_control, W_MENU_MaskPlot, /SENSITIVE
  widget_control, W_MENU_SetBad, /SENSITIVE
  widget_control, W_MENU_DetImage, /SENSITIVE
  widget_control, W_MENU_ShVlist, /SENSITIVE
  widget_control, W_MENU_PSVlist, /SENSITIVE
  widget_control, W_MENU_AfTime2Date, /SENSITIVE
  widget_control, W_MENU_SvVdet, /SENSITIVE
  widget_control, W_MENU_MkSubTSD, /SENSITIVE
  widget_control, W_MENU_FFT, /SENSITIVE
  widget_control, W_MENU_EDIT, /SENSITIVE
  widget_control, W_MENU_ClearFlag, /SENSITIVE
  widget_control, W_MENU_GBbatch, /SENSITIVE
  widget_control, W_MENU_FlgB, /SENSITIVE
  widget_control, W_MENU_RAnm, /SENSITIVE
  widget_control, W_MENU_CnvV, /SENSITIVE
  widget_control, W_MENU_RCcr, /SENSITIVE
  widget_control, W_MENU_Diff, /SENSITIVE
  widget_control, W_MENU_Peri, /SENSITIVE
  widget_control, W_MENU_MTGL, /SENSITIVE
  widget_control, W_MENU_GPGL, /SENSITIVE
  widget_control, W_MENU_MCLT, /SENSITIVE
  widget_control, W_MENU_MDKT, /SENSITIVE
  widget_control, W_MENU_DKSb, /SENSITIVE
  widget_control, W_MENU_ResC, /SENSITIVE
  widget_control, W_MENU_FlxC, /SENSITIVE

end
;-----------------------------------------------------------------
; Save TSD file
;-----------------------------------------------------------------
pro FileSave, Event
COMMON Share, tsd

  ; If TSD file has not been opened
  IF obj_valid(tsd) NE 1 THEN BEGIN
    result = DIALOG_MESSAGE("TSD file has not been opened.",$
              DIALOG_PARENT = Event.top)
    Return
  ENDIF

  ; Get the current fine name
  init_fname = tsd->Get_Filename()

  ; Remove '.gz' from a default save file name in the dialog box
  ret = strmatch(init_fname, '*.gz')
  IF (ret EQ 1) THEN BEGIN
    pos = strpos(init_fname, '.gz')
    init_fname = strmid(init_fname, 0, pos)
  ENDIF

  ; Open dialog to set a new file name
  new_tsd_filename = DIALOG_PICKFILE(FILTER = ['*.fits','*.fits.gz'], /WRITE, /OVERWRITE_PROMPT,$
              DEFAULT_EXTENSION='fits', FILE=init_fname, DIALOG_PARENT = Event.top)

  ; If any file was not selected
  IF (new_tsd_filename EQ "") THEN Return

  ; Append history to the fits header
  dummy = tsd_fxaddhist(tsd, 'Saved by FISv', /curtime, /prres)
  ; Write out
  result = tsd->write_tsd(new_tsd_filename)

  ; Get ID of Info Box
  WID_TEXT_0 = widget_info(Event.top, find_by_uname='WID_TEXT_0')
  ; Show information
  WIDGET_CONTROL, WID_TEXT_0, get_value = textvalue
  WIDGET_CONTROL, WID_TEXT_0, set_value = new_tsd_filename+' has been saveed.', $
    /append, SET_TEXT_TOP_LINE=n_elements(textvalue)-2

end
;-----------------------------------------------------------------
; Activate Button Callback Procedure.
; Argument:
;   Event structure:
;
;   {WIDGET_BUTTON, ID:0L, TOP:0L, HANDLER:0L, SELECT:0}
;
;   ID is the widget ID of the component generating the event. TOP is
;       the widget ID of the top level widget containing ID. HANDLER
;       contains the widget ID of the widget associated with the
;       handler routine.

;   SELECT is set to 1 if the button was set, and 0 if released.
;       Normal buttons do not generate events when released, so
;       SELECT will always be 1. However, toggle buttons (created by
;       parenting a button to an exclusive or non-exclusive base)
;       return separate events for the set and release actions.

;   Retrieve the IDs of other widgets in the widget hierarchy using
;       id=widget_info(Event.top, FIND_BY_UNAME=name)

;-----------------------------------------------------------------
; Exit FISv (FIS Data Visualizer)
;-----------------------------------------------------------------
pro OnExit, Event
COMMON Share

  WIDGET_CONTROL, Event.top, /DESTROY

  ; Destroy all existing pointer heap variables
  ;OBJ_DESTROY, OBJ_VALID()
  IF (OBJ_VALID(tsd)) THEN OBJ_DESTROY, tsd

end

;-----------------------------------------------------------------
; TSD FITS Header Viewer
;-----------------------------------------------------------------
pro HeaderView, Event
  tlb = widget_info(Event.top, find_by_uname='FISV_BASE_0')
  view_fits_header, GROUP_LEADER=tlb
end

;-----------------------------------------------------------------
; Show All pixels plot
;-----------------------------------------------------------------
pro ListVdetSig, Event
  tlb = widget_info(Event.top, find_by_uname='FISV_BASE_0')
  list_vdet, GROUP_LEADER=tlb
end

;-----------------------------------------------------------------
; Make PS file of all pixels plot
;-----------------------------------------------------------------
pro PSListVdetSig, Event
  list_vdet_ps
end

;-----------------------------------------------------------------
; FFT
;-----------------------------------------------------------------
pro FFT_DATA, Event

  tlb = widget_info(Event.top, find_by_uname='FISV_BASE_0')

  ; Get ID of DATA selector YTree
  WID_TREE_1 = widget_info(Event.top, find_by_uname='WID_TREE_1')
  ; Selected index in YTree
  Selected_Ytree = widget_info(WID_TREE_1, /tree_select)

  ; Get uvalue of YTree
  IF n_elements(Selected_Ytree) EQ 1 THEN BEGIN
    IF Selected_Ytree NE -1 THEN BEGIN
      widget_control, Selected_Ytree, get_uvalue=yuvalue
    ENDIF ELSE BEGIN
      ; Show error message
      result = DIALOG_MESSAGE(["No data for FFT!"],$
                  DIALOG_PARENT=Event.top, /ERROR)
      print, 'No data are selected!'
      Return
    ENDELSE
  ENDIF ELSE IF n_elements(Selected_Ytree) GT 1 THEN BEGIN
    ; Show error message
    result = DIALOG_MESSAGE(["Too many data for FFT!"],$
                DIALOG_PARENT=Event.top, /ERROR)
    print, 'Too many data are selected!'
    Return
  ENDIF

  ; Set Y data values
  xuvalue = 'INDEX'
  xyData, xuvalue, yuvalue, x, y, xtitle, ytitle, title

  data = y
  data_name = yuvalue
  fft_box, data, DNAME=data_name, GROUP_LEADER=tlb
end

;-----------------------------------------------------------------
; Call GB_Batch.pro
;-----------------------------------------------------------------
pro GB_batch_proc, Event
COMMON Share

  ; Get ID of Info Box
  WID_TEXT_0 = widget_info(Event.top, find_by_uname='WID_TEXT_0')
  ; Show information
  WIDGET_CONTROL, WID_TEXT_0, get_value = textvalue
  WIDGET_CONTROL, WID_TEXT_0, set_value = 'GB (pipeline modules) continuous processing', $
    /append, SET_TEXT_TOP_LINE=n_elements(textvalue)-2

  gb_batch, GROUP_LEADER=Event.top

end

;-----------------------------------------------------------------
; Call GB_Flag_Bad_Data.pro
;-----------------------------------------------------------------
pro GB_FlagBD, Event
COMMON Share

  ; Get ID of Info Box
  WID_TEXT_0 = widget_info(Event.top, find_by_uname='WID_TEXT_0')
  ;
  WIDGET_CONTROL, WID_TEXT_0, get_value = textvalue
  WIDGET_CONTROL, WID_TEXT_0, set_value = 'GB_Flag_Bad_Data.pro was called.', $
    /append, SET_TEXT_TOP_LINE=n_elements(textvalue)-2

  WIDGET_CONTROL, /HOURGLASS
  result = gb_run_gb(tsd, outf=-1, GB_MOD='gb_flag_bad_data')
end

;-----------------------------------------------------------------
; Call GB_Set_Flag_Rstanom.pro
;-----------------------------------------------------------------
pro GB_FlagRAnm, Event
COMMON Share

  ; Get ID of Info Box
  WID_TEXT_0 = widget_info(Event.top, find_by_uname='WID_TEXT_0')
  ;
  WIDGET_CONTROL, WID_TEXT_0, get_value = textvalue
  WIDGET_CONTROL, WID_TEXT_0, set_value = 'GB_Set_Flag_Rstanom.pro was called.', $
    /append, SET_TEXT_TOP_LINE=n_elements(textvalue)-2

  WIDGET_CONTROL, /HOURGLASS
  result = gb_run_gb(tsd, outf=-1, GB_MOD='gb_set_flag_rstanom')
end

;-----------------------------------------------------------------
; Call GB_Conv_to_Volt.pro
;-----------------------------------------------------------------
pro GB_Conv2Volt, Event
COMMON Share

  ; Get ID of Info Box
  WID_TEXT_0 = widget_info(Event.top, find_by_uname='WID_TEXT_0')
  ;
  WIDGET_CONTROL, WID_TEXT_0, get_value = textvalue
  WIDGET_CONTROL, WID_TEXT_0, set_value = 'GB_Conv_to_Volt.pro was called.', $
    /append, SET_TEXT_TOP_LINE=n_elements(textvalue)-2

  ; Ask if noise estimation will be done
  mesg = ['Execute with Noise Estimate?', $
          '(Noise estimate requires rather long time.)', $
          '', $
          'Yes    : with Noise estimate', $
          'No     : w/o  Noise estimate', $
          'Cancel : cancel GB_Conv2Volt', $
          '', $
          '(Noise values are put in FERR.)']
  ret = DIALOG_MESSAGE(mesg, /QUESTION, /CANCEL, DIALOG_PARENT=Event.top, TITLE='GB_Conv_to_Volt')

  IF (ret EQ 'Yes') THEN BEGIN
    ferr = 1
  ENDIF ELSE IF (ret EQ 'No') THEN BEGIN
    ferr = 0
  ENDIF ELSE BEGIN ; If 'Cancel'
    WIDGET_CONTROL, WID_TEXT_0, set_value = 'GB_Conv_to_Volt was canceled.', $
    /append, SET_TEXT_TOP_LINE=n_elements(textvalue)-2
    return
  ENDELSE

  WIDGET_CONTROL, /HOURGLASS
  result = gb_run_gb(tsd, outf=-1, GB_MOD='gb_conv_to_volt', C2V_FERR=ferr)
end

;-----------------------------------------------------------------
; Call GB_Ramp_Curve_corr.pro
;-----------------------------------------------------------------
pro GB_RCcrr, Event
COMMON Share

  ; Get ID of Info Box
  WID_TEXT_0 = widget_info(Event.top, find_by_uname='WID_TEXT_0')
  ;
  WIDGET_CONTROL, WID_TEXT_0, get_value = textvalue
  WIDGET_CONTROL, WID_TEXT_0, set_value = 'GB_Ramp_Curve_Corr.pro was called.', $
    /append, SET_TEXT_TOP_LINE=n_elements(textvalue)-2

;  ; Check the file of correction table for existence
;  datatype = tsd->Get_Datatype()
;  IF (datatype EQ 'FIS_SW') THEN result = file_test('rctbl_sw.ccf') $
;  ELSE IF (datatype EQ 'FIS_LW') THEN result = file_test('rctbl_lw.ccf')
;
;  IF result EQ 0 THEN BEGIN
;    ; Get current working directory
;    CD, '.', CURRENT=crt_dir
;
;    info = 'You need "rctbl_(sw|lw).ccf" in the current directory. '+'Current directory is "'+crt_dir+'".'
;    ; to Command line
;    print, info
;    ; to Info Box
;    WIDGET_CONTROL, WID_TEXT_0, get_value = textvalue
;    WIDGET_CONTROL, WID_TEXT_0, set_value = info, $
;      /append, SET_TEXT_TOP_LINE=n_elements(textvalue)-2
;    ; to Dialog
;    r = dialog_message(info, /ERROR)
;    ;
;    Return
;  ENDIF

  WIDGET_CONTROL, /HOURGLASS
  result = gb_run_gb(tsd, outf=-1, GB_MOD='gb_ramp_curve_corr', /RC_Noplot)
end

;-----------------------------------------------------------------
; Call GB_Differentiation.pro
;-----------------------------------------------------------------
pro GB_Diff, Event
COMMON Share

  ; Get ID of Info Box
  WID_TEXT_0 = widget_info(Event.top, find_by_uname='WID_TEXT_0')
  ;
  WIDGET_CONTROL, WID_TEXT_0, get_value = textvalue
  WIDGET_CONTROL, WID_TEXT_0, set_value = 'GB_Differentiation.pro was called.', $
    /append, SET_TEXT_TOP_LINE=n_elements(textvalue)-2

  ; Show confirmation message
;  result = DIALOG_MESSAGE("Do you differentiate the FLUX signals?",$
;               DIALOG_PARENT=Event.top, TITLE='Confirmation', /QUESTION)
;  IF result EQ 'No' THEN Retall

  ; Get ID of preview window
  WID_DRAW_0=widget_info(Event.top, find_by_uname='WID_DRAW_0')
  WIDGET_CONTROL, WID_DRAW_0, get_value=preview_window
  wset, preview_window

  WIDGET_CONTROL, /HOURGLASS
  result = gb_run_gb(tsd, outf=-1, GB_MOD='gb_differentiation')
end

;-----------------------------------------------------------------
; Call GB_Periodic_Noise
;-----------------------------------------------------------------
pro GB_Peri, Event
COMMON Share

  ; Get ID of Info Box
  WID_TEXT_0 = widget_info(Event.top, find_by_uname='WID_TEXT_0')
  ;
  WIDGET_CONTROL, WID_TEXT_0, get_value = textvalue
  WIDGET_CONTROL, WID_TEXT_0, set_value = 'GB_Periodic_Noise.pro was called.', $
    /append, SET_TEXT_TOP_LINE=n_elements(textvalue)-2

  WIDGET_CONTROL, /HOURGLASS
  result = gb_run_gb(tsd, outf=-1, GB_MOD='gb_periodic_noise')
end

;-----------------------------------------------------------------
; Call GB_Set_Glitch_Status_MMT.pro
;-----------------------------------------------------------------
pro GB_MTGL, Event
COMMON Share

  ; Get ID of Info Box
  WID_TEXT_0 = widget_info(Event.top, find_by_uname='WID_TEXT_0')
  ;
  WIDGET_CONTROL, WID_TEXT_0, get_value = textvalue
  WIDGET_CONTROL, WID_TEXT_0, set_value = 'GB_Set_Glitch_Status_MMT.pro was called.', $
    /append, SET_TEXT_TOP_LINE=n_elements(textvalue)-2

  WIDGET_CONTROL, /HOURGLASS
  result = gb_run_gb(tsd, outf=-1, GB_MOD='gb_set_glitch_status_mmt')
end

;-----------------------------------------------------------------
; Call GB_Set_Glitch_Status_GP.pro
;-----------------------------------------------------------------
pro GB_GPGL, Event
COMMON Share

  ; Get ID of Info Box
  WID_TEXT_0 = widget_info(Event.top, find_by_uname='WID_TEXT_0')
  ;
  WIDGET_CONTROL, WID_TEXT_0, get_value = textvalue
  WIDGET_CONTROL, WID_TEXT_0, set_value = 'GB_Set_Glitch_Status_GP.pro was called.', $
    /append, SET_TEXT_TOP_LINE=n_elements(textvalue)-2

  WIDGET_CONTROL, /HOURGLASS
  result = gb_run_gb(tsd, outf=-1, GB_MOD='gb_set_glitch_status_gp')
end

;-----------------------------------------------------------------
; Call GB_Make_DC_Cal_Lamp_Table.pro
;-----------------------------------------------------------------
pro GB_MCLT, Event
COMMON Share

  ; Get ID of Info Box
  WID_TEXT_0 = widget_info(Event.top, find_by_uname='WID_TEXT_0')
  ;
  WIDGET_CONTROL, WID_TEXT_0, get_value = textvalue
  WIDGET_CONTROL, WID_TEXT_0, set_value = 'GB_Make_DC_Cal_Lamp_Table.pro was called.', $
    /append, SET_TEXT_TOP_LINE=n_elements(textvalue)-2

  ; Ask if table files and a subdirectory may be made or not.
  CD, '.', CURRENT=crt_dir ; Get current working directory
  mesg = ['Make table files and a subdirectory(CALTABLE) in the current directory?', $
         'Current directory is "'+crt_dir+'"']
  ret = DIALOG_MESSAGE(mesg, /QUESTION, DIALOG_PARENT=Event.top, $
                       TITLE='GB_Make_DC_Cal_Lamp_Table')

  IF (ret EQ 'No') THEN BEGIN
    WIDGET_CONTROL, WID_TEXT_0, set_value = 'GB_Make_DC_Cal_Lamp_Table was canceled.', $
    /append, SET_TEXT_TOP_LINE=n_elements(textvalue)-2
    return
  ENDIF

  WIDGET_CONTROL, /HOURGLASS
  result = gb_run_gb(tsd, outf=-1, GB_MOD='gb_make_dc_cal_lamp_table')
end

;-----------------------------------------------------------------
; Call GB_Make_Dark_Table.pro
;-----------------------------------------------------------------
pro GB_MDKT, Event
COMMON Share

  ; Get ID of Info Box
  WID_TEXT_0 = widget_info(Event.top, find_by_uname='WID_TEXT_0')
  ;
  WIDGET_CONTROL, WID_TEXT_0, get_value = textvalue
  WIDGET_CONTROL, WID_TEXT_0, set_value = 'GB_Make_Dark_Table.pro was called.', $
    /append, SET_TEXT_TOP_LINE=n_elements(textvalue)-2

  ; Ask if table files and a subdirectory may be made or not.
  CD, '.', CURRENT=crt_dir ; Get current working directory
  mesg = ['Make table files and a subdirectory(DARKTABLE) in the current directory?', $
         'Current directory is "'+crt_dir+'"']
  ret = DIALOG_MESSAGE(mesg, /QUESTION, DIALOG_PARENT=Event.top, $
                       TITLE='GB_Make_Dark_Table')

  IF (ret EQ 'No') THEN BEGIN
    WIDGET_CONTROL, WID_TEXT_0, set_value = 'GB_Make_Dark_Table was canceled.', $
    /append, SET_TEXT_TOP_LINE=n_elements(textvalue)-2
    return
  ENDIF

  WIDGET_CONTROL, /HOURGLASS
  result = gb_run_gb(tsd, outf=-1, GB_MOD='gb_make_dark_table')
end

;-----------------------------------------------------------------
; Call GB_Dark_Subtraction.pro
;-----------------------------------------------------------------
pro GB_DKSb, Event
COMMON Share

  ; Get ID of Info Box
  WID_TEXT_0 = widget_info(Event.top, find_by_uname='WID_TEXT_0')
  ;
  WIDGET_CONTROL, WID_TEXT_0, get_value = textvalue
  WIDGET_CONTROL, WID_TEXT_0, set_value = 'GB_Dark_Subtraction.pro was called.', $
    /append, SET_TEXT_TOP_LINE=n_elements(textvalue)-2

  WIDGET_CONTROL, /HOURGLASS
  result = gb_run_gb(tsd, outf=-1, GB_MOD='gb_dark_subtraction')
end

;-----------------------------------------------------------------
; Call GB_DC_Responsivity_Corr.pro
;-----------------------------------------------------------------
pro GB_ResC, Event
COMMON Share

  ; Get ID of Info Box
  WID_TEXT_0 = widget_info(Event.top, find_by_uname='WID_TEXT_0')
  ;
  WIDGET_CONTROL, WID_TEXT_0, get_value = textvalue
  WIDGET_CONTROL, WID_TEXT_0, set_value = 'GB_DC_Responsivity_Corr.pro was called.', $
    /append, SET_TEXT_TOP_LINE=n_elements(textvalue)-2

  WIDGET_CONTROL, /HOURGLASS
  result = gb_run_gb(tsd, outf=-1, GB_MOD='gb_dc_responsivity_corr')
end

;-----------------------------------------------------------------
; Call GB_Flux_Calib.pro
;-----------------------------------------------------------------
pro GB_FluxC, Event
COMMON Share

  ; Get ID of Info Box
  WID_TEXT_0 = widget_info(Event.top, find_by_uname='WID_TEXT_0')
  ;
  WIDGET_CONTROL, WID_TEXT_0, get_value = textvalue
  WIDGET_CONTROL, WID_TEXT_0, set_value = 'GB_Flux_Calib.pro was called.', $
    /append, SET_TEXT_TOP_LINE=n_elements(textvalue)-2

  WIDGET_CONTROL, /HOURGLASS
  result = gb_run_gb(tsd, outf=-1, GB_MOD='gb_flux_calib')
end

;-----------------------------------------------------------------
; Activate Button Callback Procedure.
;-----------------------------------------------------------------
; Display the data plot using zplot_alt.pro
;-----------------------------------------------------------------
pro PlotBtn, Event
COMMON Share

; Get ID of DATA selector XTree
WID_TREE_0 = widget_info(Event.top, find_by_uname='WID_TREE_0')
; Selected index in XTree
Selected_Xtree = widget_info(WID_TREE_0, /tree_select)
; Get uvalue of XTree
IF Selected_Xtree NE -1 THEN $
widget_control, Selected_Xtree, get_uvalue=xuvalue
; If no uvalue is available
IF (n_elements(xuvalue) EQ 0) THEN Return

; Get ID of DATA selector YTree
WID_TREE_1 = widget_info(Event.top, find_by_uname='WID_TREE_1')
; Selected index in YTree
Selected_Ytree = widget_info(WID_TREE_1, /tree_select)
; Sort index of selected tree branch
Selected_Ytree = Selected_Ytree[sort(Selected_Ytree)]

; Discard data without valid uvalue
IF n_elements(Selected_Ytree) GT 1 THEN BEGIN
  valid_idx=-1
  FOR i=0, n_elements(Selected_Ytree)-1, 1 DO BEGIN
    widget_control, Selected_Ytree[i], get_uvalue=yuvalue
    IF (n_elements(yuvalue) NE 0) THEN valid_idx=[valid_idx, i]
  ENDFOR
  Selected_Ytree = Selected_Ytree[valid_idx[1:*]]
ENDIF
;print, Selected_Ytree;#####

yuvalue_all=''
; Get uvalue of YTree
IF n_elements(Selected_Ytree) EQ 1 THEN BEGIN
    IF Selected_Ytree NE -1 THEN BEGIN
      widget_control, Selected_Ytree, get_uvalue=yuvalue

      ; If no uvalue is available
      IF (n_elements(yuvalue) EQ 0) THEN Return

      ; Set X&Y data values, titles
      xyData, xuvalue, yuvalue, x, y, xtitle, ytitle, title
      yuvalue_all=[yuvalue]
    ENDIF

ENDIF ELSE IF n_elements(Selected_Ytree) EQ 2 THEN BEGIN
    widget_control, Selected_Ytree[0], get_uvalue=yuvalue
    widget_control, Selected_Ytree[1], get_uvalue=y2uvalue

    ; Set X&Y data values, titles
    IF (n_elements(yuvalue) NE 0) THEN BEGIN
      xyData, xuvalue, yuvalue, x, y, xtitle, ytitle, title
      yuvalue_all=[yuvalue]
    ENDIF
    IF (n_elements(y2uvalue) NE 0) THEN BEGIN
      xyData, xuvalue, y2uvalue, x, y2, xtitle, y2title, title
      yuvalue_all=[yuvalue_all,y2uvalue]
    ENDIF
    ; Discard title and Y_title
    title='' & ytitle=''

ENDIF ELSE IF n_elements(Selected_Ytree) EQ 3 THEN BEGIN
    widget_control, Selected_Ytree[0], get_uvalue=yuvalue
    widget_control, Selected_Ytree[1], get_uvalue=y2uvalue
    widget_control, Selected_Ytree[2], get_uvalue=y3uvalue
    ; Set X&Y data values, titles
    IF (n_elements(yuvalue) NE 0) THEN BEGIN
      xyData, xuvalue, yuvalue, x, y, xtitle, ytitle, title
      yuvalue_all=[yuvalue]
    ENDIF
    IF (n_elements(y2uvalue) NE 0) THEN BEGIN
      xyData, xuvalue, y2uvalue, x, y2, xtitle, y2title, title
      yuvalue_all=[yuvalue_all,y2uvalue]
    ENDIF
    IF (n_elements(y3uvalue) NE 0) THEN BEGIN
      xyData, xuvalue, y3uvalue, x, y3, xtitle, y3title, title
      yuvalue_all=[yuvalue_all,y3uvalue]
    ENDIF
    ; Discard title and Y_title
    title='' & ytitle=''

ENDIF ELSE IF n_elements(Selected_Ytree) EQ 4 THEN BEGIN
    widget_control, Selected_Ytree[0], get_uvalue=yuvalue
    widget_control, Selected_Ytree[1], get_uvalue=y2uvalue
    widget_control, Selected_Ytree[2], get_uvalue=y3uvalue
    widget_control, Selected_Ytree[3], get_uvalue=y4uvalue
    ; Set X&Y data values, titles
    IF (n_elements(yuvalue) NE 0) THEN BEGIN
      xyData, xuvalue, yuvalue, x, y, xtitle, ytitle, title
      yuvalue_all=[yuvalue]
    ENDIF
    IF (n_elements(y2uvalue) NE 0) THEN BEGIN
      xyData, xuvalue, y2uvalue, x, y2, xtitle, y2title, title
      yuvalue_all=[yuvalue_all,y2uvalue]
    ENDIF
    IF (n_elements(y3uvalue) NE 0) THEN BEGIN
      xyData, xuvalue, y3uvalue, x, y3, xtitle, y3title, title
      yuvalue_all=[yuvalue_all,y3uvalue]
    ENDIF
    IF (n_elements(y4uvalue) NE 0) THEN BEGIN
      xyData, xuvalue, y4uvalue, x, y4, xtitle, y4title, title
      yuvalue_all=[yuvalue_all,y4uvalue]
    ENDIF
    ; Discard title and Y_title
    title='' & ytitle=''

ENDIF ELSE IF n_elements(Selected_Ytree) EQ 5 THEN BEGIN
    widget_control, Selected_Ytree[0], get_uvalue=yuvalue
    widget_control, Selected_Ytree[1], get_uvalue=y2uvalue
    widget_control, Selected_Ytree[2], get_uvalue=y3uvalue
    widget_control, Selected_Ytree[3], get_uvalue=y4uvalue
    widget_control, Selected_Ytree[4], get_uvalue=y5uvalue
    ; Set X&Y data values, titles
    IF (n_elements(yuvalue) NE 0) THEN BEGIN
      xyData, xuvalue, yuvalue, x, y, xtitle, ytitle, title
      yuvalue_all=[yuvalue]
    ENDIF
    IF (n_elements(y2uvalue) NE 0) THEN BEGIN
      xyData, xuvalue, y2uvalue, x, y2, xtitle, y2title, title
      yuvalue_all=[yuvalue_all,y2uvalue]
    ENDIF
    IF (n_elements(y3uvalue) NE 0) THEN BEGIN
      xyData, xuvalue, y3uvalue, x, y3, xtitle, y3title, title
      yuvalue_all=[yuvalue_all,y3uvalue]
    ENDIF
    IF (n_elements(y4uvalue) NE 0) THEN BEGIN
      xyData, xuvalue, y4uvalue, x, y4, xtitle, y4title, title
      yuvalue_all=[yuvalue_all,y4uvalue]
    ENDIF
    IF (n_elements(y5uvalue) NE 0) THEN BEGIN
      xyData, xuvalue, y5uvalue, x, y5, xtitle, y5title, title
      yuvalue_all=[yuvalue_all,y5uvalue]
    ENDIF
    ; Discard title and Y_title
    title='' & ytitle=''

ENDIF ELSE IF n_elements(Selected_Ytree) GT 5 THEN BEGIN
    ; Show error message
    result = DIALOG_MESSAGE(["Too many data for plot!", "Max 5 data are expected."],$
                DIALOG_PARENT=Event.top, /ERROR)
    print, 'Too many data are selected!'

    ; Show error message in the message window
    WID_TEXT_0 = widget_info(Event.top, find_by_uname='WID_TEXT_0')
    WIDGET_CONTROL, WID_TEXT_0, get_value = textvalue
    WIDGET_CONTROL, WID_TEXT_0, set_value = 'Too many data are selected! Max : 5 for Y', $
      /append, SET_TEXT_TOP_LINE=n_elements(textvalue)-2
ENDIF

;help, yuvalue, y2uvalue, y3uvalue, y4uvalue, y5uvalue;#####
;help, y, y2, y3, y4, y5;#####
; If data type is 'byte', result of differentiation in zplot_alt is going to be wrong.
; So, convert data type from byte to integer.
IF (size(y  ,/type) EQ 1) THEN y =fix(y)
IF (size(y2 ,/type) EQ 1) THEN y2=fix(y2)
IF (size(y3 ,/type) EQ 1) THEN y3=fix(y3)
IF (size(y4 ,/type) EQ 1) THEN y4=fix(y4)
IF (size(y5 ,/type) EQ 1) THEN y5=fix(y5)

IF (n_elements(x) NE 0) AND (n_elements(y) NE 0) THEN BEGIN
  print, 'X:',xuvalue,', Y:',yuvalue_all

  tlb = widget_info(Event.top, find_by_uname='FISV_BASE_0')
  ; Adjustment of xmargin & tickformat of plot in case of 'aftime'
  IF (strmatch(yuvalue, 'AFTIME')) then xmargin=[17.0, 3.0] else xmargin=[12.0, 3.0]
  IF (strmatch(xuvalue, 'AFTIME')) then xtickformat='(D13.3)' else xtickformat=''
  IF (strmatch(yuvalue, 'AFTIME')) then ytickformat='(D13.3)' else ytickformat=''
  ; Plot on another window
  zplot_alt, x, y, Y2DATA=y2, Y3DATA=y3, Y4DATA=y4, Y5DATA=y5 $
    , GROUP_LEADER=tlb $
    , ZOOM_XSIZE=Float(GetEnv('ZPlot_X_SIZE')), ZOOM_YSIZE=Float(GetEnv('ZPlot_Y_SIZE')) $
    , TITLE=title, XTITLE=xtitle, YTITLE=ytitle $
    , Y1TITLE=yuvalue, Y2TITLE=y2uvalue, Y3TITLE=y3uvalue $
    , Y4TITLE=y4uvalue, Y5TITLE=y5uvalue $
    , FILENAME = tsd->get_filename() $
    , CHARSIZE = (!d.name EQ 'X') ? 1.4 : 1 $
    , XMARGIN = xmargin, XTICKFORMAT=xtickformat, YTICKFORMAT=ytickformat $
    , EXTBTN_MODE=Fix(GetEnv('EXTBTN_MODE'))

ENDIF

end

;-----------------------------------------------------------------
; Activate Button Callback Procedure.
;-----------------------------------------------------------------
; Display a window of data plot masked by pixel flags
;-----------------------------------------------------------------
pro MaskPlot, Event
COMMON Share

  ; Get ID of Info Box
  WID_TEXT_0 = widget_info(Event.top, find_by_uname='WID_TEXT_0')
  ; Show information
  WIDGET_CONTROL, WID_TEXT_0, get_value = textvalue
  WIDGET_CONTROL, WID_TEXT_0, set_value = 'Plot DET and FLUX data masked by pixel flags.', $
    /append, SET_TEXT_TOP_LINE=n_elements(textvalue)-2

  maskflag, GROUP_LEADER=Event.top
end

;-----------------------------------------------------------------
; Activate Button Callback Procedure.
;-----------------------------------------------------------------
; Set bad pixel-flags of a selected pixel
;-----------------------------------------------------------------
pro SetBad, Event
COMMON Share

  ; Get ID of Info Box
  WID_TEXT_0 = widget_info(Event.top, find_by_uname='WID_TEXT_0')
  ; Show information
  WIDGET_CONTROL, WID_TEXT_0, get_value = textvalue
  WIDGET_CONTROL, WID_TEXT_0, set_value = 'Set additional bad flag.', $
    /append, SET_TEXT_TOP_LINE=n_elements(textvalue)-2

  ;;;;
  ;;;; Check the selected data
  ;;;;

  ; Get ID of DATA selector XTree
  WID_TREE_0 = widget_info(Event.top, find_by_uname='WID_TREE_0')
  ; Selected index in XTree
  Selected_Xtree = widget_info(WID_TREE_0, /tree_select)
  ; Get uvalue of XTree
  IF Selected_Xtree NE -1 THEN $
  widget_control, Selected_Xtree, get_uvalue=xuvalue
  ; If no uvalue is available
  IF (n_elements(xuvalue) EQ 0) THEN BEGIN
    result = DIALOG_MESSAGE(['X data must be selected.'], DIALOG_PARENT=Event.top, /ERROR)
    print, 'X data must be selected.'
    RETALL
  ENDIF

  ; Get ID of DATA selector YTree
  WID_TREE_1 = widget_info(Event.top, find_by_uname='WID_TREE_1')
  ; Selected index in YTree
  Selected_Ytree = widget_info(WID_TREE_1, /tree_select)
  ; Sort index of selected tree branch
  Selected_Ytree = Selected_Ytree[sort(Selected_Ytree)]

  ; Discard data without valid uvalue
  IF n_elements(Selected_Ytree) GT 1 THEN BEGIN
    valid_idx=-1
    FOR i=0, n_elements(Selected_Ytree)-1, 1 DO BEGIN
      widget_control, Selected_Ytree[i], get_uvalue=yuvalue
      IF (n_elements(yuvalue) NE 0) THEN valid_idx=[valid_idx, i]
    ENDFOR
    Selected_Ytree = Selected_Ytree[valid_idx[1:*]]
  ENDIF

  yuvalue_all=''
  ; Get uvalue of YTree
  IF n_elements(Selected_Ytree) EQ 1 THEN BEGIN
    IF Selected_Ytree NE -1 THEN BEGIN
      widget_control, Selected_Ytree, get_uvalue=yuvalue

      ; If no yuvalue is available
      IF (n_elements(yuvalue) EQ 0) THEN BEGIN
        result = DIALOG_MESSAGE(['DET or FLUX must be selected.'], DIALOG_PARENT=Event.top, /ERROR)
        print, 'DET or FLUX must be selected.'
        RETALL
      ENDIF

      ; Set X&Y data values, titles
      xyData, xuvalue, yuvalue, x, y, xtitle, ytitle, title
      yuvalue_all=[yuvalue]

      ; If no X value is available
      IF (n_elements(x) EQ 0) THEN BEGIN
        result = DIALOG_MESSAGE(['X data must be selected.'], DIALOG_PARENT=Event.top, /ERROR)
        print, 'X data must be selected.'
        RETALL
      ENDIF
      ; If no Y value is available
      IF (n_elements(y) EQ 0) THEN BEGIN
        result = DIALOG_MESSAGE(['DET or FLUX must be selected.'], DIALOG_PARENT=Event.top, /ERROR)
        print, 'DET or FLUX must be selected.'
        RETALL
      ENDIF

      masked_x = x
      masked_y = y

    ; If no Y is selected
    ENDIF ELSE IF (Selected_Ytree EQ -1) THEN BEGIN
      result = DIALOG_MESSAGE(['DET or FLUX must be selected.'], DIALOG_PARENT=Event.top, /ERROR)
      print, 'DET or FLUX must be selected.'
      RETALL
    ENDIF

  ; If multiple Y are selected
  ENDIF ELSE IF (n_elements(Selected_Ytree) GT 1) THEN BEGIN
    ; Show error message
    result = DIALOG_MESSAGE(["Too many data are selected for Y!", "Just one pixel of DET or FLUX is expected."],$
                DIALOG_PARENT=Event.top, /ERROR)
    print, 'Too many data are selected!'

    ; Show error message in the message window
    WID_TEXT_0 = widget_info(Event.top, find_by_uname='WID_TEXT_0')
    WIDGET_CONTROL, WID_TEXT_0, get_value = textvalue
    WIDGET_CONTROL, WID_TEXT_0, set_value = 'Too many data are selected!', $
      /append, SET_TEXT_TOP_LINE=n_elements(textvalue)-2

    RETALL
  ENDIF

  ; If the data is VDET or FLUX, then get the pixel number. Eles, exit.
  IF (strmatch(yuvalue, 'VDET*') or strmatch(yuvalue, 'FLUX*') ) EQ 1 THEN BEGIN
    ch = uint(strmid(yuvalue, 4,3))
  ENDIF ELSE BEGIN
    result = DIALOG_MESSAGE(['Selected Y data must be DET or FLUX.'], DIALOG_PARENT=fisv_top_id, /ERROR)
    print, 'Selected Y data must be DET or FLUX.'
    RETALL
  ENDELSE

  ;
  IF (n_elements(xuvalue) NE 0) AND (n_elements(y) NE 0) THEN BEGIN
    print, 'X:',xuvalue,', Y:',yuvalue_all

    tlb = widget_info(Event.top, find_by_uname='FISV_BASE_0')
    ; Adjustment of xmargin & tickformat of plot in case of 'aftime'
    IF (strmatch(yuvalue, 'AFTIME')) then xmargin=[17.0, 3.0] else xmargin=[12.0, 3.0]
    IF (strmatch(xuvalue, 'AFTIME')) then xtickformat='(D13.3)' else xtickformat=''
    IF (strmatch(yuvalue, 'AFTIME')) then ytickformat='(D13.3)' else ytickformat=''

    ; x-data must not be integer because some part of them will be set
    ; as !Values.F_NaN in setbad_plot.
    masked_x = float(masked_x)

    ; Plot on another window
    setbad_plot, masked_x, masked_y $
      , GROUP_LEADER=tlb $
      , ZOOM_XSIZE=Float(GetEnv('ZPlot_X_SIZE')), ZOOM_YSIZE=Float(GetEnv('ZPlot_Y_SIZE')) $
      , TITLE=title, XTITLE=xtitle $
      , FILENAME = tsd->get_filename() $
      , CHARSIZE = (!d.name EQ 'X') ? 1.4 : 1 $
      , XMARGIN = xmargin, XTICKFORMAT=xtickformat, YTICKFORMAT=ytickformat $
      , EXTBTN_MODE=Fix(GetEnv('EXTBTN_MODE')) $
      , PIXNO=ch-1
    ENDIF

  ;;setbad_plot, PIXNO=ch-1, GROUP_LEADER=Event.top
end

;-----------------------------------------------------------------
; Activate Button Callback Procedure.
;-----------------------------------------------------------------
; Printer output of data plot
;-----------------------------------------------------------------
pro PrintBtn, Event
COMMON Share

; Get ID of DATA selector XTree
WID_TREE_0 = widget_info(Event.top, find_by_uname='WID_TREE_0')
; Selected index in XTree
Selected_Xtree = widget_info(WID_TREE_0, /tree_select)
; Get uvalue of XTree
IF Selected_Xtree NE -1 THEN $
widget_control, Selected_Xtree, get_uvalue=xuvalue

; Get ID of DATA selector YTree
WID_TREE_1 = widget_info(Event.top, find_by_uname='WID_TREE_1')
; Selected index in YTree
Selected_Ytree = widget_info(WID_TREE_1, /tree_select)
; Sort index of selected tree branch
Selected_Ytree = Selected_Ytree[sort(Selected_Ytree)]

; Discard data without valid uvalue
IF n_elements(Selected_Ytree) GT 1 THEN BEGIN
  valid_idx=-1
  FOR i=0, n_elements(Selected_Ytree)-1, 1 DO BEGIN
    widget_control, Selected_Ytree[i], get_uvalue=yuvalue
    IF (n_elements(yuvalue) NE 0) THEN valid_idx=[valid_idx, i]
  ENDFOR
  Selected_Ytree = Selected_Ytree[valid_idx[1:*]]
ENDIF
;print, Selected_Ytree;#####

yuvalue_all=''
; Get uvalue of YTree
IF n_elements(Selected_Ytree) EQ 1 THEN BEGIN
    IF Selected_Ytree NE -1 THEN BEGIN
      widget_control, Selected_Ytree, get_uvalue=yuvalue

      ; If no uvalue is available
      IF (n_elements(yuvalue) EQ 0) THEN Return

      ; Set X&Y data values, titles
      xyData, xuvalue, yuvalue, x, y, xtitle, ytitle, title
      yuvalue_all=[yuvalue]
    ENDIF

ENDIF ELSE IF n_elements(Selected_Ytree) EQ 2 THEN BEGIN
    widget_control, Selected_Ytree[0], get_uvalue=yuvalue
    widget_control, Selected_Ytree[1], get_uvalue=y2uvalue

    ; Set X&Y data values, titles
    IF (n_elements(yuvalue) NE 0) THEN BEGIN
      xyData, xuvalue, yuvalue, x, y, xtitle, ytitle, title
      yuvalue_all=[yuvalue]
    ENDIF
    IF (n_elements(y2uvalue) NE 0) THEN BEGIN
      xyData, xuvalue, y2uvalue, x, y2, xtitle, y2title, title
      yuvalue_all=[yuvalue_all,y2uvalue]
    ENDIF
    ; Discard title and Y_title
    title='' & ytitle=''

ENDIF ELSE IF n_elements(Selected_Ytree) EQ 3 THEN BEGIN
    widget_control, Selected_Ytree[0], get_uvalue=yuvalue
    widget_control, Selected_Ytree[1], get_uvalue=y2uvalue
    widget_control, Selected_Ytree[2], get_uvalue=y3uvalue
    ; Set X&Y data values, titles
    IF (n_elements(yuvalue) NE 0) THEN BEGIN
      xyData, xuvalue, yuvalue, x, y, xtitle, ytitle, title
      yuvalue_all=[yuvalue]
    ENDIF
    IF (n_elements(y2uvalue) NE 0) THEN BEGIN
      xyData, xuvalue, y2uvalue, x, y2, xtitle, y2title, title
      yuvalue_all=[yuvalue_all,y2uvalue]
    ENDIF
    IF (n_elements(y3uvalue) NE 0) THEN BEGIN
      xyData, xuvalue, y3uvalue, x, y3, xtitle, y3title, title
      yuvalue_all=[yuvalue_all,y3uvalue]
    ENDIF
    ; Discard title and Y_title
    title='' & ytitle=''

ENDIF ELSE IF n_elements(Selected_Ytree) EQ 4 THEN BEGIN
    widget_control, Selected_Ytree[0], get_uvalue=yuvalue
    widget_control, Selected_Ytree[1], get_uvalue=y2uvalue
    widget_control, Selected_Ytree[2], get_uvalue=y3uvalue
    widget_control, Selected_Ytree[3], get_uvalue=y4uvalue
    ; Set X&Y data values, titles
    IF (n_elements(yuvalue) NE 0) THEN BEGIN
      xyData, xuvalue, yuvalue, x, y, xtitle, ytitle, title
      yuvalue_all=[yuvalue]
    ENDIF
    IF (n_elements(y2uvalue) NE 0) THEN BEGIN
      xyData, xuvalue, y2uvalue, x, y2, xtitle, y2title, title
      yuvalue_all=[yuvalue_all,y2uvalue]
    ENDIF
    IF (n_elements(y3uvalue) NE 0) THEN BEGIN
      xyData, xuvalue, y3uvalue, x, y3, xtitle, y3title, title
      yuvalue_all=[yuvalue_all,y3uvalue]
    ENDIF
    IF (n_elements(y4uvalue) NE 0) THEN BEGIN
      xyData, xuvalue, y4uvalue, x, y4, xtitle, y4title, title
      yuvalue_all=[yuvalue_all,y4uvalue]
    ENDIF
    ; Discard title and Y_title
    title='' & ytitle=''

ENDIF ELSE IF n_elements(Selected_Ytree) EQ 5 THEN BEGIN
    widget_control, Selected_Ytree[0], get_uvalue=yuvalue
    widget_control, Selected_Ytree[1], get_uvalue=y2uvalue
    widget_control, Selected_Ytree[2], get_uvalue=y3uvalue
    widget_control, Selected_Ytree[3], get_uvalue=y4uvalue
    widget_control, Selected_Ytree[4], get_uvalue=y5uvalue
    ; Set X&Y data values, titles
    IF (n_elements(yuvalue) NE 0) THEN BEGIN
      xyData, xuvalue, yuvalue, x, y, xtitle, ytitle, title
      yuvalue_all=[yuvalue]
    ENDIF
    IF (n_elements(y2uvalue) NE 0) THEN BEGIN
      xyData, xuvalue, y2uvalue, x, y2, xtitle, y2title, title
      yuvalue_all=[yuvalue_all,y2uvalue]
    ENDIF
    IF (n_elements(y3uvalue) NE 0) THEN BEGIN
      xyData, xuvalue, y3uvalue, x, y3, xtitle, y3title, title
      yuvalue_all=[yuvalue_all,y3uvalue]
    ENDIF
    IF (n_elements(y4uvalue) NE 0) THEN BEGIN
      xyData, xuvalue, y4uvalue, x, y4, xtitle, y4title, title
      yuvalue_all=[yuvalue_all,y4uvalue]
    ENDIF
    IF (n_elements(y5uvalue) NE 0) THEN BEGIN
      xyData, xuvalue, y5uvalue, x, y5, xtitle, y5title, title
      yuvalue_all=[yuvalue_all,y5uvalue]
    ENDIF
    ; Discard title and Y_title
    title='' & ytitle=''

ENDIF ELSE IF n_elements(Selected_Ytree) GT 5 THEN BEGIN
    ; Show error message
    result = DIALOG_MESSAGE(["Too many data for print!", "Max 5 data are expected."],$
                DIALOG_PARENT=Event.top, /ERROR)
    print, 'Too many data are selected!'

    ; Show error message in the message window
    WID_TEXT_0 = widget_info(Event.top, find_by_uname='WID_TEXT_0')
    WIDGET_CONTROL, WID_TEXT_0, get_value = textvalue
    WIDGET_CONTROL, WID_TEXT_0, set_value = 'Too many data are selected! Max : 5 for Y', $
      /append, SET_TEXT_TOP_LINE=n_elements(textvalue)-2
ENDIF


IF (n_elements(xuvalue) NE 0) AND (n_elements(y) NE 0) THEN BEGIN
  print, 'X:',xuvalue,', Y:',yuvalue_all

  ; Printer OUTPUT
  result=dialog_printersetup()

  IF result EQ 1 THEN BEGIN
    entry_device = !d.name
    set_plot, 'PRINTER'

    ; Set YRANGE
    IF N_Elements(y2) EQ 0 THEN y2 = mean(y)
    IF N_Elements(y3) EQ 0 THEN y3 = mean(y)
    IF N_Elements(y4) EQ 0 THEN y4 = mean(y)
    IF N_Elements(y5) EQ 0 THEN y5 = mean(y)
    yrange=[Min([y,y2,y3,y4,y5]),Max([y,y2,y3,y4,y5])]

    ; Plot
    plot, x, y, TITLE=title, charsize=0.7, /ynozero, YRANGE=yrange, $
      XTITLE=xtitle, YTITLE=ytitle
    IF n_elements(y2) EQ n_elements(x) THEN $
      oplot, x, y2, linestyle=1, thick=2
    IF n_elements(y3) EQ n_elements(x) THEN $
      oplot, x, y3, linestyle=2, thick=2
    IF n_elements(y4) EQ n_elements(x) THEN $
      oplot, x, y4, linestyle=3, thick=1
    IF n_elements(y5) EQ n_elements(x) THEN $
      oplot, x, y5, linestyle=4, thick=2

    XYOUTS, 0.02, -0.02, tsd->get_filename(), /NORMAL
    XYOUTS, 0.6, 1.0, systime(), /NORMAL

    device, /close_document
    set_plot, entry_device
  ENDIF
  ; End of Printer OUTPUT

ENDIF

end

;-----------------------------------------------------------------
; Activate Button Callback Procedure.
;-----------------------------------------------------------------
; PS file output of data plot
;-----------------------------------------------------------------
pro PSBtn, Event
COMMON Share

; Get ID of DATA selector XTree
WID_TREE_0 = widget_info(Event.top, find_by_uname='WID_TREE_0')
; Selected index in XTree
Selected_Xtree = widget_info(WID_TREE_0, /tree_select)
; Get uvalue of XTree
IF Selected_Xtree NE -1 THEN $
widget_control, Selected_Xtree, get_uvalue=xuvalue

; Get ID of DATA selector YTree
WID_TREE_1 = widget_info(Event.top, find_by_uname='WID_TREE_1')
; Selected index in YTree
Selected_Ytree = widget_info(WID_TREE_1, /tree_select)
; Sort index of selected tree branch
Selected_Ytree = Selected_Ytree[sort(Selected_Ytree)]

; Discard data without valid uvalue
IF n_elements(Selected_Ytree) GT 1 THEN BEGIN
  valid_idx=-1
  FOR i=0, n_elements(Selected_Ytree)-1, 1 DO BEGIN
    widget_control, Selected_Ytree[i], get_uvalue=yuvalue
    IF (n_elements(yuvalue) NE 0) THEN valid_idx=[valid_idx, i]
  ENDFOR
  Selected_Ytree = Selected_Ytree[valid_idx[1:*]]
ENDIF
;print, Selected_Ytree;#####

yuvalue_all=''
; Get uvalue of YTree
IF n_elements(Selected_Ytree) EQ 1 THEN BEGIN
    IF Selected_Ytree NE -1 THEN BEGIN
      widget_control, Selected_Ytree, get_uvalue=yuvalue

      ; If no uvalue is available
      IF (n_elements(yuvalue) EQ 0) THEN Return

      ; Set X&Y data values, titles
      xyData, xuvalue, yuvalue, x, y, xtitle, ytitle, title
      yuvalue_all=[yuvalue]
    ENDIF

ENDIF ELSE IF n_elements(Selected_Ytree) EQ 2 THEN BEGIN
    widget_control, Selected_Ytree[0], get_uvalue=yuvalue
    widget_control, Selected_Ytree[1], get_uvalue=y2uvalue

    ; Set X&Y data values, titles
    IF (n_elements(yuvalue) NE 0) THEN BEGIN
      xyData, xuvalue, yuvalue, x, y, xtitle, ytitle, title
      yuvalue_all=[yuvalue]
    ENDIF
    IF (n_elements(y2uvalue) NE 0) THEN BEGIN
      xyData, xuvalue, y2uvalue, x, y2, xtitle, y2title, title
      yuvalue_all=[yuvalue_all,y2uvalue]
    ENDIF
    ; Discard title and Y_title
    title='' & ytitle=''

ENDIF ELSE IF n_elements(Selected_Ytree) EQ 3 THEN BEGIN
    widget_control, Selected_Ytree[0], get_uvalue=yuvalue
    widget_control, Selected_Ytree[1], get_uvalue=y2uvalue
    widget_control, Selected_Ytree[2], get_uvalue=y3uvalue
    ; Set X&Y data values, titles
    IF (n_elements(yuvalue) NE 0) THEN BEGIN
      xyData, xuvalue, yuvalue, x, y, xtitle, ytitle, title
      yuvalue_all=[yuvalue]
    ENDIF
    IF (n_elements(y2uvalue) NE 0) THEN BEGIN
      xyData, xuvalue, y2uvalue, x, y2, xtitle, y2title, title
      yuvalue_all=[yuvalue_all,y2uvalue]
    ENDIF
    IF (n_elements(y3uvalue) NE 0) THEN BEGIN
      xyData, xuvalue, y3uvalue, x, y3, xtitle, y3title, title
      yuvalue_all=[yuvalue_all,y3uvalue]
    ENDIF
    ; Discard title and Y_title
    title='' & ytitle=''

ENDIF ELSE IF n_elements(Selected_Ytree) EQ 4 THEN BEGIN
    widget_control, Selected_Ytree[0], get_uvalue=yuvalue
    widget_control, Selected_Ytree[1], get_uvalue=y2uvalue
    widget_control, Selected_Ytree[2], get_uvalue=y3uvalue
    widget_control, Selected_Ytree[3], get_uvalue=y4uvalue
    ; Set X&Y data values, titles
    IF (n_elements(yuvalue) NE 0) THEN BEGIN
      xyData, xuvalue, yuvalue, x, y, xtitle, ytitle, title
      yuvalue_all=[yuvalue]
    ENDIF
    IF (n_elements(y2uvalue) NE 0) THEN BEGIN
      xyData, xuvalue, y2uvalue, x, y2, xtitle, y2title, title
      yuvalue_all=[yuvalue_all,y2uvalue]
    ENDIF
    IF (n_elements(y3uvalue) NE 0) THEN BEGIN
      xyData, xuvalue, y3uvalue, x, y3, xtitle, y3title, title
      yuvalue_all=[yuvalue_all,y3uvalue]
    ENDIF
    IF (n_elements(y4uvalue) NE 0) THEN BEGIN
      xyData, xuvalue, y4uvalue, x, y4, xtitle, y4title, title
      yuvalue_all=[yuvalue_all,y4uvalue]
    ENDIF
    ; Discard title and Y_title
    title='' & ytitle=''

ENDIF ELSE IF n_elements(Selected_Ytree) EQ 5 THEN BEGIN
    widget_control, Selected_Ytree[0], get_uvalue=yuvalue
    widget_control, Selected_Ytree[1], get_uvalue=y2uvalue
    widget_control, Selected_Ytree[2], get_uvalue=y3uvalue
    widget_control, Selected_Ytree[3], get_uvalue=y4uvalue
    widget_control, Selected_Ytree[4], get_uvalue=y5uvalue
    ; Set X&Y data values, titles
    IF (n_elements(yuvalue) NE 0) THEN BEGIN
      xyData, xuvalue, yuvalue, x, y, xtitle, ytitle, title
      yuvalue_all=[yuvalue]
    ENDIF
    IF (n_elements(y2uvalue) NE 0) THEN BEGIN
      xyData, xuvalue, y2uvalue, x, y2, xtitle, y2title, title
      yuvalue_all=[yuvalue_all,y2uvalue]
    ENDIF
    IF (n_elements(y3uvalue) NE 0) THEN BEGIN
      xyData, xuvalue, y3uvalue, x, y3, xtitle, y3title, title
      yuvalue_all=[yuvalue_all,y3uvalue]
    ENDIF
    IF (n_elements(y4uvalue) NE 0) THEN BEGIN
      xyData, xuvalue, y4uvalue, x, y4, xtitle, y4title, title
      yuvalue_all=[yuvalue_all,y4uvalue]
    ENDIF
    IF (n_elements(y5uvalue) NE 0) THEN BEGIN
      xyData, xuvalue, y5uvalue, x, y5, xtitle, y5title, title
      yuvalue_all=[yuvalue_all,y5uvalue]
    ENDIF
    ; Discard title and Y_title
    title='' & ytitle=''

ENDIF ELSE IF n_elements(Selected_Ytree) GT 5 THEN BEGIN
    ; Show error message
    result = DIALOG_MESSAGE(["Too many data for PS!", "Max 5 data are expected."],$
                DIALOG_PARENT=Event.top, /ERROR)
    print, 'Too many data are selected!'

    ; Show error message in the message window
    WID_TEXT_0 = widget_info(Event.top, find_by_uname='WID_TEXT_0')
    WIDGET_CONTROL, WID_TEXT_0, get_value = textvalue
    WIDGET_CONTROL, WID_TEXT_0, set_value = 'Too many data are selected! Max : 5 for Y', $
      /append, SET_TEXT_TOP_LINE=n_elements(textvalue)-2
ENDIF


IF (n_elements(xuvalue) NE 0) AND (n_elements(y) NE 0) THEN BEGIN
  print, 'X:',xuvalue,', Y:',yuvalue_all

  ; PS file OUTPUT
  ; Using FSC_PSConfig program
  ;psObject = Obj_New("FSC_PSConfig", Color=0, /European)
  ;psObject->GUI, Cancel=cancelled, /FontInfo
  psObject = Obj_New("FSC_PSConfig", /Color, /European, /Landscape, $
                     XSize=20.0, YSize=15.0, XOffset=3.0029, YOffset=24.859)
  psObject->GUI, Cancel=cancelled, /FontInfo, Accept_Button_Name='OK'

  IF cancelled EQ 0 THEN BEGIN
    entry_device = !d.name
    set_plot, 'PS'
    ;device, filename="testOUT.ps"
    device, _Extra=psObject->GetKeywords()

    ; Set YRANGE
    IF N_Elements(y2) EQ 0 THEN y2 = mean(y)
    IF N_Elements(y3) EQ 0 THEN y3 = mean(y)
    IF N_Elements(y4) EQ 0 THEN y4 = mean(y)
    IF N_Elements(y5) EQ 0 THEN y5 = mean(y)
    yrange=[Min([y,y2,y3,y4,y5]),Max([y,y2,y3,y4,y5])]

    ; Plot
    plot, x, y, TITLE=title, charsize=0.7, /ynozero, YRANGE=yrange, $
      XTITLE=xtitle, YTITLE=ytitle
    IF n_elements(y2) EQ n_elements(x) THEN $
      oplot, x, y2, linestyle=1, thick=2
    IF n_elements(y3) EQ n_elements(x) THEN $
      oplot, x, y3, linestyle=2, thick=2
    IF n_elements(y4) EQ n_elements(x) THEN $
      oplot, x, y4, linestyle=3, thick=1
    IF n_elements(y5) EQ n_elements(x) THEN $
      oplot, x, y5, linestyle=4, thick=2

    XYOUTS, 0.02, -0.02, tsd->get_filename(), /NORMAL
    XYOUTS, 0.6, 1.0, systime(), /NORMAL

    device, /close_file
    set_plot, entry_device
  ENDIF

  Obj_Destroy, psObject
  ; End of PS file OUTPUT

ENDIF

end

;-----------------------------------------------------------------
; Activate Button Callback Procedure.
;-----------------------------------------------------------------
; Demonstrate the detector 2D image
;-----------------------------------------------------------------
pro DetImage, Event
COMMON Share

  det_image, GROUP_LEADER=Event.top

end

;-----------------------------------------------------------------
; Tree Select Callback Procedure.
; Argument:
;   Event structure:
;
;   {WIDGET_TREE_SEL, ID:0L, TOP:0L, HANDLER:0L, TYPE:0, CLICKS:0L}
;   ID is the widget ID of the component generating the event. TOP is
;       the widget ID of the top level widget containing ID. HANDLER
;       contains the widget ID of the widget associated with the
;       handler routine.

;   The CLICKS field contains either 1 or 2, depending upon how the
;       tree item was selected. If the tree item is double-clicked,
;       CLICKS is set to 2.

;   Retrieve the IDs of other widgets in the widget hierarchy using
;       id=widget_info(Event.top, FIND_BY_UNAME=name)

;-----------------------------------------------------------------
; Select X-axis data and preview
;-----------------------------------------------------------------
pro XDataSelected, Event
COMMON Share

; Get uvalue of XTree
widget_control, Event.id, get_uvalue=xuvalue
IF (n_elements(xuvalue) EQ 0) THEN Return

; Make child trees
CASE xuvalue of

  'XStatus': BEGIN
    if widget_info(Event.id,/child) eq 0 then begin
      make_x_status_tree, Event.id
      widget_control, Event.id, /set_tree_expanded
      Return
    endif else begin
      Return
    endelse
  END
  'XAnlgTele': BEGIN
    if widget_info(Event.id,/child) eq 0 then begin
      make_x_anlgtele_tree, Event.id
      widget_control, Event.id, /set_tree_expanded
      Return
    endif else begin
      Return
    endelse
  END
  'XVdet': BEGIN
    if widget_info(Event.id,/child) eq 0 then begin
      make_x_vdet_tree, Event.id
      widget_control, Event.id, /set_tree_expanded
      Return
    endif else begin
      Return
    endelse
  END
  'XFlux': BEGIN
    if widget_info(Event.id,/child) eq 0 then begin
      make_x_flux_tree, Event.id
      widget_control, Event.id, /set_tree_expanded
      Return
    endif else begin
      Return
    endelse
  END
  'XFerr': BEGIN
    if widget_info(Event.id,/child) eq 0 then begin
      make_x_ferr_tree, Event.id
      widget_control, Event.id, /set_tree_expanded
      Return
    endif else begin
      Return
    endelse
  END
  'XFlag': BEGIN
    if widget_info(Event.id,/child) eq 0 then begin
      make_x_pixflag_tree, Event.id
      widget_control, Event.id, /set_tree_expanded
      Return
    endif else begin
      Return
    endelse
  END
  'XPixFlag': BEGIN
    if widget_info(Event.id,/child) eq 0 then begin
      Widget_Control, Event.id, Get_VALUE=pixflag_type
      make_x_pixflag_ch_tree, Event.id, pixflag_type
      widget_control, Event.id, /set_tree_expanded
      Return
    endif else begin
      Return
    endelse
  END
  'XQual': BEGIN
    if widget_info(Event.id,/child) eq 0 then begin
      make_x_quality_tree, Event.id
      widget_control, Event.id, /set_tree_expanded
      Return
    endif else begin
      Return
    endelse
  END
  'XQuality': BEGIN
    if widget_info(Event.id,/child) eq 0 then begin
      Widget_Control, Event.id, Get_VALUE=quality_type
      make_x_quality_ch_tree, Event.id, quality_type
      widget_control, Event.id, /set_tree_expanded
      Return
    endif else begin
      Return
    endelse
  END
  'XCunt': BEGIN
    if widget_info(Event.id,/child) eq 0 then begin
      make_x_counter_tree, Event.id
      widget_control, Event.id, /set_tree_expanded
      Return
    endif else begin
      Return
    endelse
  END
  'XCounter': BEGIN
    if widget_info(Event.id,/child) eq 0 then begin
      Widget_Control, Event.id, Get_VALUE=counter_type
      make_x_counter_ch_tree, Event.id, counter_type
      widget_control, Event.id, /set_tree_expanded
      Return
    endif else begin
      Return
    endelse
  END
  'X_FIS_HK': BEGIN
    if widget_info(Event.id,/child) eq 0 then begin
      make_x_fis_hk_tree, Event.id
      widget_control, Event.id, /set_tree_expanded
      Return
    endif else begin
      Return
    endelse
  END
  'X_IRC_HK': BEGIN
    if widget_info(Event.id,/child) eq 0 then begin
      make_x_irc_hk_tree, Event.id
      widget_control, Event.id, /set_tree_expanded
      Return
    endif else begin
      Return
    endelse
  END
  'X_HK_2': BEGIN
    if widget_info(Event.id,/child) eq 0 then begin
      make_x_hk_2_tree, Event.id
      widget_control, Event.id, /set_tree_expanded
      Return
    endif else begin
      Return
    endelse
  END
  'X_AOCU': BEGIN
    if widget_info(Event.id,/child) eq 0 then begin
      make_x_aocu_tree, Event.id
      widget_control, Event.id, /set_tree_expanded
      Return
    endif else begin
      Return
    endelse
  END
  'X_GADS': BEGIN
    if widget_info(Event.id,/child) eq 0 then begin
      make_x_gads_tree, Event.id
      widget_control, Event.id, /set_tree_expanded
      Return
    endif else begin
      Return
    endelse
  END
  'X_PR': BEGIN
    if widget_info(Event.id,/child) eq 0 then begin
      make_x_pr_tree, Event.id
      widget_control, Event.id, /set_tree_expanded
      Return
    endif else begin
      Return
    endelse
  END
  'X_SE': BEGIN
    if widget_info(Event.id,/child) eq 0 then begin
      make_x_se_tree, Event.id
      widget_control, Event.id, /set_tree_expanded
      Return
    endif else begin
      Return
    endelse
  END

  ELSE:
ENDCASE

; Get ID of DATA selector YTree
WID_TREE_1 = widget_info(Event.top, find_by_uname='WID_TREE_1')
; Selected index in YTree
Selected_Ytree = widget_info(WID_TREE_1, /tree_select)
; Get uvalue of YTree
IF n_elements(Selected_Ytree) EQ 1 THEN BEGIN
  IF Selected_Ytree NE -1 THEN $
    widget_control, Selected_Ytree, get_uvalue=yuvalue
ENDIF ELSE IF n_elements(Selected_Ytree) GT 1 THEN BEGIN
  widget_control, Selected_Ytree[n_elements(Selected_Ytree)-1], get_uvalue=yuvalue
ENDIF

; If yuvalue is not available, then return.
IF (n_elements(yuvalue) EQ 0) THEN Return
; If the selected Y is a folder, just return.
UVAL_mem_for_noact = ['YStatus','YAnlgTele','YVdet','YFlux','YFerr','YFlag','YPixFlag', $
                      'YQual','YQuality', 'YCunt', 'YCounter', $
                      'Y_FIS_HK','Y_IRC_HK','Y_HK_2','Y_AOCU','Y_GADS','Y_SE']
IF (total(strmatch(UVAL_mem_for_noact, yuvalue, /fold_case))) NE 0.0 THEN Return


; If X&Y UVALUE is availble
IF (n_elements(xuvalue) NE 0) AND (n_elements(yuvalue) NE 0) THEN BEGIN

  ; Show name of the X&Y data which was selected
  WID_TEXT_0 = widget_info(Event.top, find_by_uname='WID_TEXT_0')
  WIDGET_CONTROL, WID_TEXT_0, get_value = textvalue
  WIDGET_CONTROL, WID_TEXT_0, set_value = 'X:'+xuvalue+', Y:'+yuvalue, $
    /append, SET_TEXT_TOP_LINE=n_elements(textvalue)-2

  ; Set X&Y data values, titles
  xyData, xuvalue, yuvalue, x, y, xtitle, ytitle, title

  ; Get ID of preview window
  WID_DRAW_0=widget_info(Event.top, find_by_uname='WID_DRAW_0')
  WIDGET_CONTROL, WID_DRAW_0, get_value=preview_window
  wset, preview_window


  ; Set up colors for preview plot.
  backColor = GetColor('charcoal', !D.Table_Size-2)
  dataColor = GetColor('Light Yellow', !D.Table_Size-3)
  plotColor = GetColor('Gold', !D.Table_Size-4)

  ; Preview Plot
  IF (min(x) EQ max(x)) THEN BEGIN
    print, "X has constant value."
    plot, x, y, TITLE=title, charsize=0.7, /ynozero, $
      xstyle=3, background=backColor, color=dataColor, /nodata
    oplot, x, y, color=plotColor
  ENDIF ELSE BEGIN
    ;help, x,y; ## for Debug
    plot, x, y, TITLE=title, charsize=0.7, /ynozero, $
      xstyle=3, xticks=2, background=backColor, color=dataColor, /nodata
    oplot, x, y, color=plotColor
  ENDELSE

ENDIF

end

;-----------------------------------------------------------------
; Tree Select Callback Procedure.
;-----------------------------------------------------------------
; Select Y-axis data and preview
;-----------------------------------------------------------------
pro YDataSelected, Event
COMMON Share

; Get uvalue of YTree
widget_control, Event.id, get_uvalue=yuvalue
IF (n_elements(yuvalue) EQ 0) THEN Return

; Make child trees
CASE yuvalue of

  'YStatus': BEGIN
    if widget_info(Event.id,/child) eq 0 then begin
      make_y_status_tree, Event.id
      widget_control, Event.id, /set_tree_expanded
      Return
    endif else begin
      Return
    endelse
  END
  'YAnlgTele': BEGIN
    if widget_info(Event.id,/child) eq 0 then begin
      make_y_anlgtele_tree, Event.id
      widget_control, Event.id, /set_tree_expanded
      Return
    endif else begin
      Return
    endelse
  END
  'YVdet': BEGIN
    if widget_info(Event.id,/child) eq 0 then begin
      make_y_vdet_tree, Event.id
      widget_control, Event.id, /set_tree_expanded
      Return
    endif else begin
      Return
    endelse
  END
  'YFlux': BEGIN
    if widget_info(Event.id,/child) eq 0 then begin
      make_y_flux_tree, Event.id
      widget_control, Event.id, /set_tree_expanded
      Return
    endif else begin
      Return
    endelse
  END
  'YFerr': BEGIN
    if widget_info(Event.id,/child) eq 0 then begin
      make_y_ferr_tree, Event.id
      widget_control, Event.id, /set_tree_expanded
      Return
    endif else begin
      Return
    endelse
  END
  'YFlag': BEGIN
    if widget_info(Event.id,/child) eq 0 then begin
      make_y_pixflag_tree, Event.id
      widget_control, Event.id, /set_tree_expanded
      Return
    endif else begin
      Return
    endelse
  END
  'YPixFlag': BEGIN
    if widget_info(Event.id,/child) eq 0 then begin
      Widget_Control, Event.id, Get_VALUE=pixflag_type
      make_y_pixflag_ch_tree, Event.id, pixflag_type
      widget_control, Event.id, /set_tree_expanded
      Return
    endif else begin
      Return
    endelse
  END
  'YQual': BEGIN
    if widget_info(Event.id,/child) eq 0 then begin
      make_y_quality_tree, Event.id
      widget_control, Event.id, /set_tree_expanded
      Return
    endif else begin
      Return
    endelse
  END
  'YQuality': BEGIN
    if widget_info(Event.id,/child) eq 0 then begin
      Widget_Control, Event.id, Get_VALUE=quality_type
      make_y_quality_ch_tree, Event.id, quality_type
      widget_control, Event.id, /set_tree_expanded
      Return
    endif else begin
      Return
    endelse
  END
  'YCunt': BEGIN
    if widget_info(Event.id,/child) eq 0 then begin
      make_y_counter_tree, Event.id
      widget_control, Event.id, /set_tree_expanded
      Return
    endif else begin
      Return
    endelse
  END
  'YCounter': BEGIN
    if widget_info(Event.id,/child) eq 0 then begin
      Widget_Control, Event.id, Get_VALUE=counter_type
      make_y_counter_ch_tree, Event.id, counter_type
      widget_control, Event.id, /set_tree_expanded
      Return
    endif else begin
      Return
    endelse
  END
  'Y_FIS_HK': BEGIN
    if widget_info(Event.id,/child) eq 0 then begin
      make_y_fis_hk_tree, Event.id
      widget_control, Event.id, /set_tree_expanded
      Return
    endif else begin
      Return
    endelse
  END
  'Y_IRC_HK': BEGIN
    if widget_info(Event.id,/child) eq 0 then begin
      make_y_irc_hk_tree, Event.id
      widget_control, Event.id, /set_tree_expanded
      Return
    endif else begin
      Return
    endelse
  END
  'Y_HK_2': BEGIN
    if widget_info(Event.id,/child) eq 0 then begin
      make_y_hk_2_tree, Event.id
      widget_control, Event.id, /set_tree_expanded
      Return
    endif else begin
      Return
    endelse
  END
  'Y_AOCU': BEGIN
    if widget_info(Event.id,/child) eq 0 then begin
      make_y_aocu_tree, Event.id
      widget_control, Event.id, /set_tree_expanded
      Return
    endif else begin
      Return
    endelse
  END
  'Y_GADS': BEGIN
    if widget_info(Event.id,/child) eq 0 then begin
      make_y_gads_tree, Event.id
      widget_control, Event.id, /set_tree_expanded
      Return
    endif else begin
      Return
    endelse
  END
  'Y_PR': BEGIN
    if widget_info(Event.id,/child) eq 0 then begin
      make_y_pr_tree, Event.id
      widget_control, Event.id, /set_tree_expanded
      Return
    endif else begin
      Return
    endelse
  END
  'Y_SE': BEGIN
    if widget_info(Event.id,/child) eq 0 then begin
      make_y_se_tree, Event.id
      widget_control, Event.id, /set_tree_expanded
      Return
    endif else begin
      Return
    endelse
  END

  ELSE:
ENDCASE

; Get ID of DATA selector XTree
WID_TREE_0 = widget_info(Event.top, find_by_uname='WID_TREE_0')
; Selected index in XTree
Selected_Xtree = widget_info(WID_TREE_0, /tree_select)
; Get uvalue of XTree
IF Selected_Xtree NE -1 THEN $
widget_control, Selected_Xtree, get_uvalue=xuvalue

; If xuvalue is not available, then return.
IF (n_elements(xuvalue) EQ 0) THEN Return
; If the selected X is a folder, just return.
UVAL_mem_for_noact = ['XStatus','XAnlgTele','XVdet','XFlux','XFerr','XFlag','XPixFlag', $
                      'XQual','XQuality', 'XCunt', 'XCounter', $
                      'X_FIS_HK','X_IRC_HK','X_HK_2','X_AOCU','X_GADS','X_SE']
IF (total(strmatch(UVAL_mem_for_noact, xuvalue, /fold_case))) NE 0.0 THEN Return


; If X&Y UVALUE is availble
IF (n_elements(xuvalue) NE 0) AND (n_elements(yuvalue) NE 0) THEN BEGIN

  ; Show name of the X&Y data which was selected
  WID_TEXT_0 = widget_info(Event.top, find_by_uname='WID_TEXT_0')
  WIDGET_CONTROL, WID_TEXT_0, get_value = textvalue
  WIDGET_CONTROL, WID_TEXT_0, set_value = 'X:'+xuvalue+', Y:'+yuvalue, $
    /append, SET_TEXT_TOP_LINE=n_elements(textvalue)-2

  ; Set X&Y data values, titles
  xyData, xuvalue, yuvalue, x, y, xtitle, ytitle, title

  ; Get ID of preview window
  WID_DRAW_0=widget_info(Event.top, find_by_uname='WID_DRAW_0')
  WIDGET_CONTROL, WID_DRAW_0, get_value=preview_window
  wset, preview_window

  ; Set up colors for preview plot.
  backColor = GetColor('Charcoal', !D.Table_Size-2)
  dataColor = GetColor('Light Yellow', !D.Table_Size-3)
  plotColor = GetColor('Gold', !D.Table_Size-4)

  ; Preview Plot
  IF (min(x) EQ max(x)) THEN BEGIN
    print, "X has constant value."
    plot, x, y, TITLE=title, charsize=0.7, /ynozero, $
      xstyle=3, background=backColor, color=dataColor, /nodata
    oplot, x, y, color=plotColor
  ENDIF ELSE BEGIN
    ;help, x,y; ## for Debug
    plot, x, y, TITLE=title, charsize=0.7, /ynozero, $
      xstyle=3, xticks=2, background=backColor, color=dataColor, /nodata
    oplot, x, y, color=plotColor
  ENDELSE

ENDIF

end

;-----------------------------------------------------------------
; Activate Button Callback Procedure.
; Argument:
;   Event structure:
;
;   {WIDGET_BUTTON, ID:0L, TOP:0L, HANDLER:0L, SELECT:0}
;
;   ID is the widget ID of the component generating the event. TOP is
;       the widget ID of the top level widget containing ID. HANDLER
;       contains the widget ID of the widget associated with the
;       handler routine.

;   SELECT is set to 1 if the button was set, and 0 if released.
;       Normal buttons do not generate events when released, so
;       SELECT will always be 1. However, toggle buttons (created by
;       parenting a button to an exclusive or non-exclusive base)
;       return separate events for the set and release actions.

;   Retrieve the IDs of other widgets in the widget hierarchy using
;       id=widget_info(Event.top, FIND_BY_UNAME=name)

;-----------------------------------------------------------------
; Display the x&y data in a text table
;-----------------------------------------------------------------
pro TableBtn, Event
COMMON Share

; Get ID of DATA selector XTree
WID_TREE_0 = widget_info(Event.top, find_by_uname='WID_TREE_0')
; Selected index in XTree
Selected_Xtree = widget_info(WID_TREE_0, /tree_select)
; Get uvalue of XTree
IF Selected_Xtree NE -1 THEN $
  widget_control, Selected_Xtree, get_uvalue=xuvalue

; If no uvalue is available
IF (n_elements(xuvalue) EQ 0) THEN Return

; If the selected X is a folder, just return.
UVAL_mem_for_noact = ['XStatus','XAnlgTele','XVdet','XFlux','XFerr','XFlag','XPixFlag', $
                      'XQual','XQuality', 'XCunt', 'XCounter', $
                      'X_FIS_HK','X_IRC_HK','X_HK_2','X_AOCU','X_GADS','X_SE']
IF (total(strmatch(UVAL_mem_for_noact, xuvalue, /fold_case))) NE 0.0 THEN Return


; Get ID of DATA selector YTree
WID_TREE_1 = widget_info(Event.top, find_by_uname='WID_TREE_1')
; Selected index in YTree
Selected_Ytree = widget_info(WID_TREE_1, /tree_select)

; Get uvalue of YTree
; For single selection of YTree
IF n_elements(Selected_Ytree) EQ 1 THEN BEGIN
  IF Selected_Ytree NE -1 THEN BEGIN
    widget_control, Selected_Ytree, get_uvalue=yuvalue
    ; If no uvalue is available
    IF (n_elements(yuvalue) EQ 0) THEN Return

    ; If the selected Y is a folder, just return.
    UVAL_mem_for_noact = ['YStatus','YAnlgTele','YVdet','YFlux','YFerr','YFlag','YPixFlag', $
                          'YQual','YQuality', 'YCunt', 'YCounter', $
                          'Y_FIS_HK','Y_IRC_HK','Y_HK_2','Y_AOCU','Y_GADS','Y_SE']
    IF (total(strmatch(UVAL_mem_for_noact, yuvalue, /fold_case))) NE 0.0 THEN Return

    ; Set X&Y data values, titles
    xyData, xuvalue, yuvalue, x, y, xtitle, ytitle, title
  ENDIF

; For multiple selection of YTree
ENDIF ELSE IF n_elements(Selected_Ytree) GT 1 THEN BEGIN
  ; Sort index of selected tree branch
  Selected_Ytree = Selected_Ytree[sort(Selected_Ytree)]

  ; Make arrays of Y-data & Y-uvalue
  FOR i=0, n_elements(Selected_Ytree)-1, 1 DO BEGIN
    widget_control, Selected_Ytree[i], get_uvalue=yuvalue_temp

    ; If no uvalue is available
    IF (n_elements(yuvalue_temp) EQ 0) THEN Continue

    ; If the selected Y is a folder, then continue.
    UVAL_mem_for_noact = ['YStatus','YAnlgTele','YVdet','YFlux','YFerr','YFlag','YPixFlag', $
                          'YQual','YQuality', 'YCunt', 'YCounter', $
                          'Y_FIS_HK','Y_IRC_HK','Y_HK_2','Y_AOCU','Y_GADS','Y_SE']
    IF (total(strmatch(UVAL_mem_for_noact, yuvalue_temp, /fold_case))) NE 0.0 THEN Continue

    ; Set X&Y data values, titles
    IF (n_elements(yuvalue_temp) NE 0) THEN BEGIN
      xyData, xuvalue, yuvalue_temp, x, y_temp, xtitle, ytitle, title

      IF (n_elements(y) EQ 0) THEN BEGIN
        y = y_temp
        yuvalue = yuvalue_temp
      ENDIF ELSE BEGIN
        y = [[y],[y_temp]]
        yuvalue = [[yuvalue],[yuvalue_temp]]
      ENDELSE

    ENDIF
  ENDFOR

ENDIF

IF (n_elements(xuvalue) NE 0) AND (n_elements(yuvalue) NE 0) THEN BEGIN

  print, 'X:',xuvalue,', Y:',(size(yuvalue,/n_dimensions) NE 0) ? transpose(yuvalue) : yuvalue

  ; Make new table window
  TextDump, x, y, xuvalue, yuvalue, Event

ENDIF

end

;-----------------------------------------------------------------
; Convert AFTime to Date
;-----------------------------------------------------------------
pro CnvAfTime2Date, Event

  aftime2date, GROUP_LEADER=Event.top

end

;-----------------------------------------------------------------
; Extract and Save DET/FLUX (of all pixels) as a text file
;-----------------------------------------------------------------
pro SaveVdet, Event
COMMON Share, tsd

  ; Get ID of Info Box
  WID_TEXT_0 = widget_info(Event.top, find_by_uname='WID_TEXT_0')
  ; Show information
  WIDGET_CONTROL, WID_TEXT_0, get_value = textvalue
  WIDGET_CONTROL, WID_TEXT_0, set_value = 'This tool can extract DET or FLUX data and save them as a text file.', $
    /append, SET_TEXT_TOP_LINE=n_elements(textvalue)-2

  save_vdet, GROUP_LEADER=Event.top

end

;-----------------------------------------------------------------
; Make a subset from the current TSD
;-----------------------------------------------------------------
pro Mk_SubTSD, Event
COMMON Share, tsd

  ; Get ID of Info Box
  WID_TEXT_0 = widget_info(Event.top, find_by_uname='WID_TEXT_0')
  ; Show information
  WIDGET_CONTROL, WID_TEXT_0, get_value = textvalue
  WIDGET_CONTROL, WID_TEXT_0, set_value = ['This tool make a subset from the current TSD.',$
                                       'The current TSD will be replaced by the subset.'], $
    /append, SET_TEXT_TOP_LINE=n_elements(textvalue)-2

  Mk_Sub_TSD, GROUP_LEADER=Event.top

end

;-----------------------------------------------------------------
; TSD Editor
;-----------------------------------------------------------------
pro TSD_EDIT, Event
COMMON Share

; Get ID of DATA selector XTree
WID_TREE_0 = widget_info(Event.top, find_by_uname='WID_TREE_0')
; Selected index in XTree
Selected_Xtree = widget_info(WID_TREE_0, /tree_select)
; Get uvalue of XTree
IF Selected_Xtree NE -1 THEN $
  widget_control, Selected_Xtree, get_uvalue=xuvalue

; If no uvalue is available
IF (n_elements(xuvalue) EQ 0) THEN Return

; If the selected X is a folder, just return.
UVAL_mem_for_noact = ['XStatus','XAnlgTele','XVdet','XFlux','XFerr','XFlag','XPixFlag', $
                      'XQual','XQuality', 'XCunt', 'XCounter', $
                      'X_FIS_HK','X_IRC_HK','X_HK_2','X_AOCU','X_GADS','X_SE']
IF (total(strmatch(UVAL_mem_for_noact, xuvalue, /fold_case))) NE 0.0 THEN Return


; Get ID of DATA selector YTree
WID_TREE_1 = widget_info(Event.top, find_by_uname='WID_TREE_1')
; Selected index in YTree
Selected_Ytree = widget_info(WID_TREE_1, /tree_select)

; Get ID of DATA selector YTree
WID_TREE_1 = widget_info(Event.top, find_by_uname='WID_TREE_1')
; Selected index in YTree
Selected_Ytree = widget_info(WID_TREE_1, /tree_select)

; Get uvalue of YTree
; For single selection of YTree
IF n_elements(Selected_Ytree) EQ 1 THEN BEGIN
  IF Selected_Ytree NE -1 THEN BEGIN
    widget_control, Selected_Ytree, get_uvalue=yuvalue
    ; If no uvalue is available
    IF (n_elements(yuvalue) EQ 0) THEN Return

    ; If the selected Y is a folder, just return.
    UVAL_mem_for_noact = ['YStatus','YAnlgTele','YVdet','YFlux','YFerr','YFlag','YPixFlag', $
                          'YQual','YQuality', 'YCunt', 'YCounter', $
                          'Y_FIS_HK','Y_IRC_HK','Y_HK_2','Y_AOCU','Y_GADS','Y_SE']
    IF (total(strmatch(UVAL_mem_for_noact, yuvalue, /fold_case))) NE 0.0 THEN Return


    ; If no uvalue is available
    IF (n_elements(yuvalue) EQ 0) THEN Return
    ; Set X&Y data values, titles
    xyData, xuvalue, yuvalue, x, y, xtitle, ytitle, title
  ENDIF

; For multiple selection of YTree
ENDIF ELSE IF n_elements(Selected_Ytree) GT 1 THEN BEGIN
  ; Sort index of selected tree branch
  Selected_Ytree = Selected_Ytree[sort(Selected_Ytree)]
  ; Make arrays of Y-data & Y-uvalue
  FOR i=0, n_elements(Selected_Ytree)-1, 1 DO BEGIN
    widget_control, Selected_Ytree[i], get_uvalue=yuvalue_temp

    ; If no uvalue is available
    IF (n_elements(yuvalue_temp) EQ 0) THEN Continue

    ; If the selected Y is a folder, then continue.
    UVAL_mem_for_noact = ['YStatus','YAnlgTele','YVdet','YFlux','YFerr','YFlag','YPixFlag', $
                          'YQual','YQuality', 'YCunt', 'YCounter', $
                          'Y_FIS_HK','Y_IRC_HK','Y_HK_2','Y_AOCU','Y_GADS','Y_SE']
    IF (total(strmatch(UVAL_mem_for_noact, yuvalue_temp, /fold_case))) NE 0.0 THEN Continue


    ; Set X&Y data values, titles
    IF (n_elements(yuvalue_temp) NE 0) THEN BEGIN
      xyData, xuvalue, yuvalue_temp, x, y_temp, xtitle, ytitle, title

      IF (n_elements(y) EQ 0) THEN BEGIN
        y = y_temp
        yuvalue = yuvalue_temp
      ENDIF ELSE BEGIN
        y = [[y],[y_temp]]
        yuvalue = [[yuvalue],[yuvalue_temp]]
      ENDELSE

    ENDIF
  ENDFOR
ENDIF

IF (n_elements(xuvalue) NE 0) AND (n_elements(yuvalue) NE 0) THEN BEGIN

  print, 'X:',xuvalue,', Y:',(size(yuvalue,/n_dimensions) NE 0) ? transpose(yuvalue) : yuvalue

  ; Get ID of Info Box
  WID_TEXT_0 = widget_info(Event.top, find_by_uname='WID_TEXT_0')
  ; Show information
  WIDGET_CONTROL, WID_TEXT_0, get_value = textvalue
  WIDGET_CONTROL, WID_TEXT_0, set_value = 'You can edit data values in the FIS_OBS branch.', $
    /append, SET_TEXT_TOP_LINE=n_elements(textvalue)-2

  ; Make new table window
  tsdeditor, x, y, xuvalue, yuvalue, Event

ENDIF

end

;-----------------------------------------------------------------
; Clear all flags and qualities in the current TSD
;-----------------------------------------------------------------
pro Clear_Flag, Event
COMMON Share, tsd

  msg = ["Clear all flags and qualities in the 'FIS_OBS' branch?"]
  ret = DIALOG_MESSAGE(msg, /CENTER, DIALOG_PARENT=Event.top, $
        /Question, TITLE='Clear Flags')

  WIDGET_CONTROL, /HOURGLASS
  IF (ret EQ 'Yes') THEN Clear_All_Flag, tsd ELSE return

  ; Get ID of Info Box
  WID_TEXT_0 = widget_info(Event.top, find_by_uname='WID_TEXT_0')
  ; Show information
  WIDGET_CONTROL, WID_TEXT_0, get_value = textvalue
  WIDGET_CONTROL, WID_TEXT_0, set_value = ["All flags and qualities in the 'FIS_OBS' branch have been cleard up."], $
    /append, SET_TEXT_TOP_LINE=n_elements(textvalue)-2

end

;-----------------------------------------------------------------
; Help Message
;-----------------------------------------------------------------
pro OnLineHelp, Event

  msg = ['Access to Online Help at' , $
         'http://thyme.ir.isas.jaxa.jp/AF/FISvHelp/']

  ret = DIALOG_MESSAGE(msg, /CENTER, DIALOG_PARENT=Event.top, $
        /INFORMATION, TITLE='Help')

end

;-----------------------------------------------------------------
; Edit FISv environment file
;-----------------------------------------------------------------
pro EditEnvFile, Event

   envfile = './set_env_fisv.pro' ; First, look for a file in the current directory
   ftest = file_test(envfile)
   IF (ftest EQ 0) THEN BEGIN ; look for a file under the 'FISDR_ROOT'

     fisdr_root = GetEnv('FISDR_ROOT')
     IF (fisdr_root EQ '') THEN BEGIN
       ret = DIALOG_MESSAGE('Cannot edit environment file.', /CENTER, $
          DIALOG_PARENT=Event.top, /Error)
       return
     ENDIF

     envfile = fisdr_root+'/fisv/set_env_fisv.pro'
     ftest = file_test(envfile)

     IF (ftest EQ 0) THEN BEGIN
       ret = DIALOG_MESSAGE('Cannot find environment file.', /CENTER, $
          DIALOG_PARENT=Event.top, /Error)
       return
     ENDIF

   ENDIF

   ret = DIALOG_MESSAGE('The new settings are reflected after the FISv is recompiled and rebooted.', $
           /CENTER, DIALOG_PARENT=Event.top, /INFORMATION)

   XDISPLAYFILE, envfile, /EDITABLE, $
     DONE_BUTTON='Done', GROUP=Event.top, /GROW_TO_SCREEN

end

;-----------------------------------------------------------------
; About
;-----------------------------------------------------------------
pro ShowAbout, Event

  msg = ['FISv - AKARI FIS Data (TSD) Visualizer -' , $
         'Copyright (c) 2003-2007, AKARI Data Reduction Team.']

  ret = DIALOG_MESSAGE(msg, /CENTER, DIALOG_PARENT=Event.top, $
        /INFORMATION, TITLE='About FISv')

end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Empty stub procedure used for autoloading.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro fisv_eventcb
end
