; $Id: make_x_gads_tree.pro,v 1.5 2007/02/05 07:16:44 makiuti Exp $
;+
; NAME:
;       MAKE_X_GADS_TREE
;
; PURPOSE:
;       This procedure makes the GADS extension branch
;       in the X data selector of the FISv.
;       This procedure is called by FISv.
;
; AUTHOR:
;       ASTRO-F Data Reduction team
;       Web Page: http://www.ir.isas.jaxa.jp/ASTRO-F/iris_data/
;
; CATEGORY:
;       FIS Data Visualizer
;
; CALLING SEQUENCE:
;       make_x_gads_tree, widgetID
;
; MODIFICATION HISTORY:
;        Written by: Makiuti,S.
;        2007/02/05  Last modified
;-
pro make_x_gads_tree, widgetID

  parentID = widgetID
  WHILE Widget_Info(parentID, /Parent) NE 0 DO BEGIN
     parentID = Widget_Info(parentID, /Parent)
  ENDWHILE

  Widget_Control, parentID, UPDATE=0

  ;  ## ADS Extension Root
  X_TREENODE_GADS = widget_info(parentID, find_by_uname='X_TREENODE_GADS')

  ; (1) Time
  X_TREENODE_GADS_1 = Widget_Tree(X_TREENODE_GADS, UNAME='X_TREENODE_GADS_1'  $
      ,/FOLDER ,VALUE='Time')
    X_TREE_AFTIME = Widget_Tree(X_TREENODE_GADS_1, UNAME='X_TREE_GADS_AFTIME'  $
      ,VALUE='AFTIME', UVALUE='GADS_AFTIME')
    X_TREE_PIMTIORG = Widget_Tree(X_TREENODE_GADS_1, UNAME='X_TREE_GADS_PIMTIORG'  $
      ,VALUE='PIMTIORG', UVALUE='GADS_PIM_TI_ORG')

  ; (2) Status
  X_TREENODE_GADS_2 = Widget_Tree(X_TREENODE_GADS, UNAME='X_TREENODE_GADS_2'  $
      ,/FOLDER ,VALUE='Status')
    X_TREE_GADS_BLANK = Widget_Tree(X_TREENODE_GADS_2, UNAME='X_TREE_GADS_BLANK'  $
      ,VALUE='Blank', UVALUE='GADS_BLANK')

  ; (3) Analog Data
  X_TREENODE_GADS_3 = Widget_Tree(X_TREENODE_GADS, UNAME='X_TREENODE_GADS_3'  $
      ,/FOLDER ,VALUE='Analg Data')

    X_TREENODE_GADS_QUATERNION = Widget_Tree(X_TREENODE_GADS_3, UNAME='X_TREENODE_GADS_QUATERNION'  $
        ,/FOLDER ,VALUE='Attitude (Quaternion)')
      X_TREE_GADS_Q1 = Widget_Tree(X_TREENODE_GADS_QUATERNION, UNAME='X_TREE_GADS_Q1'  $
      ,VALUE='Attitude_1', UVALUE='GADS_QUAT1')
      X_TREE_GADS_Q2 = Widget_Tree(X_TREENODE_GADS_QUATERNION, UNAME='X_TREE_GADS_Q2'  $
      ,VALUE='Attitude_2', UVALUE='GADS_QUAT2')
      X_TREE_GADS_Q3 = Widget_Tree(X_TREENODE_GADS_QUATERNION, UNAME='X_TREE_GADS_Q3'  $
      ,VALUE='Attitude_3', UVALUE='GADS_QUAT3')
      X_TREE_GADS_Q4 = Widget_Tree(X_TREENODE_GADS_QUATERNION, UNAME='X_TREE_GADS_Q4'  $
      ,VALUE='Attitude_4', UVALUE='GADS_QUAT4')

    X_TREENODE_GADS_POS_ERR = Widget_Tree(X_TREENODE_GADS_3, UNAME='X_TREENODE_GADS_POS_ERR'  $
        ,/FOLDER ,VALUE='Attitude_Er')
      X_TREE_GADS_POS_ERR1 = Widget_Tree(X_TREENODE_GADS_POS_ERR, UNAME='X_TREE_GADS_POS_ERR1'  $
      ,VALUE='Attitude_Er_1', UVALUE='GADS_ERR1')
      X_TREE_GADS_POS_ERR2 = Widget_Tree(X_TREENODE_GADS_POS_ERR, UNAME='X_TREE_GADS_POS_ERR2'  $
      ,VALUE='Attitude_Er_2', UVALUE='GADS_ERR2')
      X_TREE_GADS_POS_ERR3 = Widget_Tree(X_TREENODE_GADS_POS_ERR, UNAME='X_TREE_GADS_POS_ERR3'  $
      ,VALUE='Attitude_Er_3', UVALUE='GADS_ERR3')

    X_TREENODE_GADS_ANGL_VEL = Widget_Tree(X_TREENODE_GADS_3, UNAME='X_TREENODE_ANGL_VEL'  $
        ,/FOLDER ,VALUE='Angl_Vel')
      X_TREE_GADS_BR1 = Widget_Tree(X_TREENODE_GADS_ANGL_VEL, UNAME='X_TREE_GADS_BR1'  $
      ,VALUE='Angl_Vel_1', UVALUE='GADS_BR1')
      X_TREE_GADS_BR2 = Widget_Tree(X_TREENODE_GADS_ANGL_VEL, UNAME='X_TREE_GADS_BR2'  $
      ,VALUE='Angl_Vel_2', UVALUE='GADS_BR2')
      X_TREE_GADS_BR3 = Widget_Tree(X_TREENODE_GADS_ANGL_VEL, UNAME='X_TREE_GADS_BR3'  $
      ,VALUE='Angl_Vel_3', UVALUE='GADS_BR3')

    X_TREE_GADS_BRA = Widget_Tree(X_TREENODE_GADS_3, UNAME='X_TREE_GADS_RA'  $
      ,VALUE='RA', UVALUE='GADS_RA')
    X_TREE_GADS_BDEC = Widget_Tree(X_TREENODE_GADS_3, UNAME='X_TREE_GADS_DEC'  $
      ,VALUE='DEC', UVALUE='GADS_DEC')
    X_TREE_GADS_ANG = Widget_Tree(X_TREENODE_GADS_3, UNAME='X_TREE_GADS_RANG'  $
      ,VALUE='Roll', UVALUE='GADS_RANG')

    X_TREE_GADS_DBRA = Widget_Tree(X_TREENODE_GADS_3, UNAME='X_TREE_GADS_DRA'  $
      ,VALUE='d_RA', UVALUE='GADS_DRA')
    X_TREE_GADS_DBDEC = Widget_Tree(X_TREENODE_GADS_3, UNAME='X_TREE_GADS_DDEC'  $
      ,VALUE='d_DEC', UVALUE='GADS_DDEC')
    X_TREE_GADS_DANG = Widget_Tree(X_TREENODE_GADS_3, UNAME='X_TREE_GADS_DRANG'  $
      ,VALUE='d_Roll', UVALUE='GADS_DRANG')

    X_TREE_GADS_EL_SOL = Widget_Tree(X_TREENODE_GADS_3, UNAME='X_TREE_GADS_EL_SOL'  $
      ,VALUE='EL_Sol', UVALUE='GADS_EL_SOL')
    X_TREE_GADS_AA_SOL = Widget_Tree(X_TREENODE_GADS_3, UNAME='X_TREE_GADS_AA_SOL'  $
      ,VALUE='AA_Sol', UVALUE='GADS_AA_SOL')
    X_TREE_GADS_AA_EAR = Widget_Tree(X_TREENODE_GADS_3, UNAME='X_TREE_GADS_AA_EAR'  $
      ,VALUE='AA_Ear', UVALUE='GADS_AA_EAR')
    X_TREE_GADS_AA_LUN = Widget_Tree(X_TREENODE_GADS_3, UNAME='X_TREE_GADS_AA_LUN'  $
      ,VALUE='AA_Lun', UVALUE='GADS_AA_LUN')

    X_TREE_GADS_CRS_OFF = Widget_Tree(X_TREENODE_GADS_3, UNAME='X_TREE_GADS_CRS_OFF'  $
      ,VALUE='Crs_Off', UVALUE='GADS_CRS_OFF')

  ; (4) Quality
  X_TREENODE_GADS_4 = Widget_Tree(X_TREENODE_GADS, UNAME='X_TREENODE_GADS_4'  $
      ,/FOLDER ,VALUE='Quality')
    X_TREE_GADS_QUAL_INTP = Widget_Tree(X_TREENODE_GADS_4, UNAME='X_TREE_GADS_QUAL_INTP'  $
      ,VALUE='Interpolated', UVALUE='GADS_QUAL_INTP')

  Widget_Control, parentID, /UPDATE

end