;+
; NAME:
;       MAKE_X_PR_TREE
;
; PURPOSE:
;       This procedure makes the PR extension branch
;       in the X data selector of the FISv.
;       This procedure is called by FISv.
;
; AUTHOR:
;       ASTRO-F Data Reduction team
;       Web Page: http://www.ir.isas.jaxa.jp/ASTRO-F/iris_data/
;
; CATEGORY:
;       FIS Data Visualizer
;
; CALLING SEQUENCE:
;       make_x_pr_tree, widgetID
;
; MODIFICATION HISTORY:
;        Written by: Makiuti,S.
;        2007/02/05  Last modified
;-
pro make_x_pr_tree, widgetID

  parentID = widgetID
  WHILE Widget_Info(parentID, /Parent) NE 0 DO BEGIN
     parentID = Widget_Info(parentID, /Parent)
  ENDWHILE

  Widget_Control, parentID, UPDATE=0

  ;  ## PR Extension Root
  X_TREENODE_PR = widget_info(parentID, find_by_uname='X_TREENODE_PR')

  ; (1) Time
  X_TREENODE_PR_1 = Widget_Tree(X_TREENODE_PR, UNAME='X_TREENODE_PR_1'  $
      ,/FOLDER ,VALUE='Time')
    X_TREE_AFTIME = Widget_Tree(X_TREENODE_PR_1, UNAME='X_TREE_PR_AFTIME'  $
      ,VALUE='AFTIME', UVALUE='PR_AFTIME')
    X_TREE_PIMTIORG = Widget_Tree(X_TREENODE_PR_1, UNAME='X_TREE_PR_PIMTIORG'  $
      ,VALUE='PIMTIORG', UVALUE='PR_PIM_TI_ORG')

  ; (2) Status
  X_TREENODE_PR_2 = Widget_Tree(X_TREENODE_PR, UNAME='X_TREENODE_PR_2'  $
      ,/FOLDER ,VALUE='Status')
    X_TREE_PR_BLANK = Widget_Tree(X_TREENODE_PR_2, UNAME='X_TREE_PR_BLANK'  $
      ,VALUE='Blank', UVALUE='PR_BLANK')

  ; (3) Analog Data
  X_TREENODE_PR_3 = Widget_Tree(X_TREENODE_PR, UNAME='X_TREENODE_PR_3'  $
      ,/FOLDER ,VALUE='Analg Data')

    X_TREENODE_PR_QUATERNION = Widget_Tree(X_TREENODE_PR_3, UNAME='X_TREENODE_PR_QUATERNION'  $
        ,/FOLDER ,VALUE='Quaternion')
      X_TREE_PR_Q1 = Widget_Tree(X_TREENODE_PR_QUATERNION, UNAME='X_TREE_PR_Q1'  $
      ,VALUE='Quaternion_1', UVALUE='PR_QUAT1')
      X_TREE_PR_Q2 = Widget_Tree(X_TREENODE_PR_QUATERNION, UNAME='X_TREE_PR_Q2'  $
      ,VALUE='Quaternion_2', UVALUE='PR_QUAT2')
      X_TREE_PR_Q3 = Widget_Tree(X_TREENODE_PR_QUATERNION, UNAME='X_TREE_PR_Q3'  $
      ,VALUE='Quaternion_3', UVALUE='PR_QUAT3')
      X_TREE_PR_Q4 = Widget_Tree(X_TREENODE_PR_QUATERNION, UNAME='X_TREE_PR_Q4'  $
      ,VALUE='Quaternion_4', UVALUE='PR_QUAT4')

    X_TREENODE_PR_POS_ERR = Widget_Tree(X_TREENODE_PR_3, UNAME='X_TREENODE_PR_POS_ERR'  $
        ,/FOLDER ,VALUE='Pos_Err')
      X_TREE_PR_POS_ERR1 = Widget_Tree(X_TREENODE_PR_POS_ERR, UNAME='X_TREE_PR_POS_ERR1'  $
      ,VALUE='Pos_Err_1', UVALUE='PR_ERR1')
      X_TREE_PR_POS_ERR2 = Widget_Tree(X_TREENODE_PR_POS_ERR, UNAME='X_TREE_PR_POS_ERR2'  $
      ,VALUE='Pos_Err_2', UVALUE='PR_ERR2')
      X_TREE_PR_POS_ERR3 = Widget_Tree(X_TREENODE_PR_POS_ERR, UNAME='X_TREE_PR_POS_ERR3'  $
      ,VALUE='Pos_Err_3', UVALUE='PR_ERR3')

    X_TREENODE_PR_BODY_RATE = Widget_Tree(X_TREENODE_PR_3, UNAME='X_TREENODE_PR_BODY_RATE'  $
        ,/FOLDER ,VALUE='Body_Rate')
      X_TREE_PR_BR1 = Widget_Tree(X_TREENODE_PR_BODY_RATE, UNAME='X_TREE_PR_BR1'  $
      ,VALUE='Body_Rate_1', UVALUE='PR_BR1')
      X_TREE_PR_BR2 = Widget_Tree(X_TREENODE_PR_BODY_RATE, UNAME='X_TREE_PR_BR2'  $
      ,VALUE='Body_Rate_2', UVALUE='PR_BR2')
      X_TREE_PR_BR3 = Widget_Tree(X_TREENODE_PR_BODY_RATE, UNAME='X_TREE_PR_BR3'  $
      ,VALUE='Body_Rate_3', UVALUE='PR_BR3')

    X_TREE_PR_BRA = Widget_Tree(X_TREENODE_PR_3, UNAME='X_TREE_PR_RA'  $
      ,VALUE='RA', UVALUE='PR_RA')
    X_TREE_PR_BDEC = Widget_Tree(X_TREENODE_PR_3, UNAME='X_TREE_PR_DEC'  $
      ,VALUE='DEC', UVALUE='PR_DEC')
    X_TREE_PR_ANG = Widget_Tree(X_TREENODE_PR_3, UNAME='X_TREE_PR_RANG'  $
      ,VALUE='Roll', UVALUE='PR_RANG')

    X_TREE_PR_DRA = Widget_Tree(X_TREENODE_PR_3, UNAME='X_TREE_PR_DRA'  $
      ,VALUE='d_RA', UVALUE='PR_DRA')
    X_TREE_PR_DDEC = Widget_Tree(X_TREENODE_PR_3, UNAME='X_TREE_PR_DDEC'  $
      ,VALUE='d_DEC', UVALUE='PR_DDEC')
    X_TREE_PR_DRANG = Widget_Tree(X_TREENODE_PR_3, UNAME='X_TREE_PR_DRANG'  $
      ,VALUE='d_Roll', UVALUE='PR_DRANG')

    X_TREE_PR_EL_SOL = Widget_Tree(X_TREENODE_PR_3, UNAME='X_TREE_PR_EL_SOL'  $
      ,VALUE='EL_Sol', UVALUE='PR_EL_SOL')
    X_TREE_PR_AA_SOL = Widget_Tree(X_TREENODE_PR_3, UNAME='X_TREE_PR_AA_SOL'  $
      ,VALUE='AA_Sol', UVALUE='PR_AA_SOL')
    X_TREE_PR_AA_EAR = Widget_Tree(X_TREENODE_PR_3, UNAME='X_TREE_PR_AA_EAR'  $
      ,VALUE='AA_Ear', UVALUE='PR_AA_EAR')
    X_TREE_PR_AA_LUN = Widget_Tree(X_TREENODE_PR_3, UNAME='X_TREE_PR_AA_LUN'  $
      ,VALUE='AA_Lun', UVALUE='PR_AA_LUN')

    X_TREE_PR_CRS_OFF = Widget_Tree(X_TREENODE_PR_3, UNAME='X_TREE_PR_CRS_OFF'  $
      ,VALUE='Crs_Off', UVALUE='PR_CRS_OFF')

  ; (4) Quality
  X_TREENODE_PR_4 = Widget_Tree(X_TREENODE_PR, UNAME='X_TREENODE_PR_4'  $
      ,/FOLDER ,VALUE='Quality')
    X_TREE_PR_QUAL_INTP = Widget_Tree(X_TREENODE_PR_4, UNAME='X_TREE_PR_QUAL_INTP'  $
      ,VALUE='Interpolated', UVALUE='PR_QUAL_INTP')

  Widget_Control, parentID, /UPDATE

end