;+
; NAME:
;       MAKE_X_SE_TREE
;
; PURPOSE:
;       This procedure makes the SE extension branch
;       in the X data selector of the FISv.
;       This procedure is called by FISv.
;
; AUTHOR:
;       ASTRO-F Data Reduction team
;       Web Page: http://www.ir.isas.ac.jp/ASTRO-F/iris_data/
;
; CATEGORY:
;       FIS Data Visualizer
;
; CALLING SEQUENCE:
;       make_x_se_tree, widgetID
;
; MODIFICATION HISTORY:
;        Written by: Makiuti,S.
;        2006/12/11  Last modified
;-
pro make_x_se_tree, widgetID

  parentID = widgetID
  WHILE Widget_Info(parentID, /Parent) NE 0 DO BEGIN
     parentID = Widget_Info(parentID, /Parent)
  ENDWHILE

  Widget_Control, parentID, UPDATE=0

  ;  ## SE Extension Root
  X_TREENODE_SE = widget_info(parentID, find_by_uname='X_TREENODE_SE')

  ; (1) Time
  X_TREENODE_SE_1 = Widget_Tree(X_TREENODE_SE, UNAME='X_TREENODE_SE_1'  $
      ,/FOLDER ,VALUE='Time')
    X_TREE_AFTIME = Widget_Tree(X_TREENODE_SE_1, UNAME='X_TREE_SE_AFTIME'  $
      ,VALUE='AFTIME', UVALUE='SE_AFTIME')
    X_TREE_PIMTIORG = Widget_Tree(X_TREENODE_SE_1, UNAME='X_TREE_SE_PIMTIORG'  $
      ,VALUE='PIMTIORG', UVALUE='SE_PIM_TI_ORG')

  ; (2) Status
  X_TREENODE_SE_2 = Widget_Tree(X_TREENODE_SE, UNAME='X_TREENODE_SE_2'  $
      ,/FOLDER ,VALUE='Status')
    X_TREE_SE_BLANK = Widget_Tree(X_TREENODE_SE_2, UNAME='X_TREE_SE_BLANK'  $
      ,VALUE='Blank', UVALUE='SE_BLANK')
    X_TREE_SE_TYPE = Widget_Tree(X_TREENODE_SE_2, UNAME='X_TREE_SE_TYPE'  $
      ,VALUE='Type', UVALUE='SE_TYPE')
    X_TREE_SE_DAYNIGHT = Widget_Tree(X_TREENODE_SE_2, UNAME='X_TREE_SE_DAYNIGHT'  $
      ,VALUE='DayNight', UVALUE='SE_DAYNIGHT')
    X_TREE_SE_IN_SAA = Widget_Tree(X_TREENODE_SE_2, UNAME='X_TREE_SE_IN_SAA'  $
      ,VALUE='In_SAA', UVALUE='SE_IN_SAA')
    X_TREE_SE_IN_POLAR = Widget_Tree(X_TREENODE_SE_2, UNAME='X_TREE_SE_IN_POLAR'  $
      ,VALUE='In_Polar', UVALUE='SE_IN_POLAR')

  ; (3) Analog Data
  X_TREENODE_SE_3 = Widget_Tree(X_TREENODE_SE, UNAME='X_TREENODE_SE_3'  $
      ,/FOLDER ,VALUE='Analg Data')
    X_TREE_SE_TM_SAA = Widget_Tree(X_TREENODE_SE_3, UNAME='X_TREE_TM_SAA'  $
      ,VALUE='TM_SAA', UVALUE='SE_TM_SAA')

    X_TREE_SE_SAT_X = Widget_Tree(X_TREENODE_SE_3, UNAME='X_TREE_SE_SAT_X'  $
      ,VALUE='SAT_PosX', UVALUE='SE_SAT_X')
    X_TREE_SE_SAT_Y = Widget_Tree(X_TREENODE_SE_3, UNAME='X_TREE_SE_SAT_Y'  $
      ,VALUE='SAT_PosY', UVALUE='SE_SAT_Y')
    X_TREE_SE_SAT_Z = Widget_Tree(X_TREENODE_SE_3, UNAME='X_TREE_SE_SAT_Z'  $
      ,VALUE='SAT_PosZ', UVALUE='SE_SAT_Z')

  Widget_Control, parentID, /UPDATE

end