; $Id: make_y_aocu_tree.pro,v 1.6 2007/02/05 07:16:44 makiuti Exp $
;+
; NAME:
;       MAKE_Y_AOCU_TREE
;
; PURPOSE:
;       This procedure makes the AOCU extension branch
;       in the Y data selector of the FISv.
;       This procedure is called by FISv.
;
; AUTHOR:
;       ASTRO-F Data Reduction team
;       Web Page: http://www.ir.isas.jaxa.jp/ASTRO-F/iris_data/
;
; CATEGORY:
;       FIS Data Visualizer
;
; CALLING SEQUENCE:
;       make_y_aocu_tree, widgetID
;
; MODIFICATION HISTORY:
;        Written by: Makiuti,S.
;        2007/02/05  Last modified
;-
pro make_y_aocu_tree, widgetID

  parentID = widgetID
  WHILE Widget_Info(parentID, /Parent) NE 0 DO BEGIN
     parentID = Widget_Info(parentID, /Parent)
  ENDWHILE

  Widget_Control, parentID, UPDATE=0

  ;  ## AOCU Extension Root
  Y_TREENODE_AOCU = widget_info(parentID, find_by_uname='Y_TREENODE_AOCU')

  ; (1) Time
  Y_TREENODE_AOCU_1 = Widget_Tree(Y_TREENODE_AOCU, UNAME='Y_TREENODE_AOCU_1'  $
      ,/FOLDER ,VALUE='Time')
    Y_TREE_AFTIME = Widget_Tree(Y_TREENODE_AOCU_1, UNAME='Y_TREE_AOCU_AFTIME'  $
      ,VALUE='AFTIME', UVALUE='AOCU_AFTIME')
    Y_TREE_PIMTIORG = Widget_Tree(Y_TREENODE_AOCU_1, UNAME='Y_TREE_AOCU_PIMTIORG'  $
      ,VALUE='PIMTIORG', UVALUE='AOCU_PIM_TI_ORG')

  ; (2) AOCU Status
  Y_TREENODE_AOCU_2 = Widget_Tree(Y_TREENODE_AOCU, UNAME='Y_TREENODE_AOCU_2'  $
      ,/FOLDER ,VALUE='Status')
    Y_TREE_AOCU_BLANK = Widget_Tree(Y_TREENODE_AOCU_2, UNAME='Y_TREE_AOCU_BLANK'  $
      ,VALUE='Blank', UVALUE='AOCU_BLANK')
    Y_TREE_AOCU_AO_ROM_RAM = Widget_Tree(Y_TREENODE_AOCU_2, UNAME='Y_TREE_AOCU_AO_ROM_RAM'  $
      ,VALUE='AO_ROM_RAM', UVALUE='AOCU_AO_ROM_RAM')
    Y_TREE_AOCU_ADS_MODE_B1 = Widget_Tree(Y_TREENODE_AOCU_2, UNAME='Y_TREE_AOCU_ADS_MODE_B1'  $
      ,VALUE='ADS_Mode_B1', UVALUE='AOCU_ADS_MODE_B1')
    Y_TREE_AOCU_ADS_MODE_B0 = Widget_Tree(Y_TREENODE_AOCU_2, UNAME='Y_TREE_AOCU_ADS_MODE_B0'  $
      ,VALUE='ADS_Mode_B0', UVALUE='AOCU_ADS_MODE_B0')

  ; (3) Analog Data
  Y_TREENODE_AOCU_3 = Widget_Tree(Y_TREENODE_AOCU, UNAME='Y_TREENODE_AOCU_3'  $
      ,/FOLDER ,VALUE='Analg Data')

    Y_TREENODE_AOCU_ADS_Q = Widget_Tree(Y_TREENODE_AOCU_3, UNAME='Y_TREENODE_AOCU_ADS_Q'  $
        ,/FOLDER ,VALUE='AOCU_ADS_Q')
      Y_TREE_AOCU_ADS_Q1 = Widget_Tree(Y_TREENODE_AOCU_ADS_Q, UNAME='Y_TREE_AOCU_ADS_Q1'  $
      ,VALUE='Quaternion_1', UVALUE='ADS_Q1')
      Y_TREE_AOCU_ADS_Q2 = Widget_Tree(Y_TREENODE_AOCU_ADS_Q, UNAME='Y_TREE_AOCU_ADS_Q2'  $
      ,VALUE='Quaternion_2', UVALUE='ADS_Q2')
      Y_TREE_AOCU_ADS_Q3 = Widget_Tree(Y_TREENODE_AOCU_ADS_Q, UNAME='Y_TREE_AOCU_ADS_Q3'  $
      ,VALUE='Quaternion_3', UVALUE='ADS_Q3')
      Y_TREE_AOCU_ADS_Q4 = Widget_Tree(Y_TREENODE_AOCU_ADS_Q, UNAME='Y_TREE_AOCU_ADS_Q4'  $
      ,VALUE='Quaternion_4', UVALUE='ADS_Q4')

    Y_TREENODE_AOCU_BODY_RATE = Widget_Tree(Y_TREENODE_AOCU_3, UNAME='Y_TREENODE_AOCU_BODY_RATE'  $
        ,/FOLDER ,VALUE='AOCU_Body_Rate')
      Y_TREE_AOCU_BR1 = Widget_Tree(Y_TREENODE_AOCU_BODY_RATE, UNAME='Y_TREE_AOCU_BR1'  $
      ,VALUE='Body_Rate_1', UVALUE='AOCU_BR1')
      Y_TREE_AOCU_BR2 = Widget_Tree(Y_TREENODE_AOCU_BODY_RATE, UNAME='Y_TREE_AOCU_BR2'  $
      ,VALUE='Body_Rate_2', UVALUE='AOCU_BR2')
      Y_TREE_AOCU_BR3 = Widget_Tree(Y_TREENODE_AOCU_BODY_RATE, UNAME='Y_TREE_AOCU_BR3'  $
      ,VALUE='Body_Rate_3', UVALUE='AOCU_BR3')

    Y_TREE_AOCU_BRA = Widget_Tree(Y_TREENODE_AOCU_3, UNAME='Y_TREE_AOCU_RA'  $
      ,VALUE='RA', UVALUE='AOCU_RA')
    Y_TREE_AOCU_BDEC = Widget_Tree(Y_TREENODE_AOCU_3, UNAME='Y_TREE_AOCU_DEC'  $
      ,VALUE='DEC', UVALUE='AOCU_DEC')
    Y_TREE_AOCU_ANG = Widget_Tree(Y_TREENODE_AOCU_3, UNAME='Y_TREE_AOCU_RANG'  $
      ,VALUE='Roll', UVALUE='AOCU_RANG')

    Y_TREE_AOCU_DBRA = Widget_Tree(Y_TREENODE_AOCU_3, UNAME='Y_TREE_AOCU_DRA'  $
      ,VALUE='d_RA', UVALUE='AOCU_DRA')
    Y_TREE_AOCU_DBDEC = Widget_Tree(Y_TREENODE_AOCU_3, UNAME='Y_TREE_AOCU_DDEC'  $
      ,VALUE='d_DEC', UVALUE='AOCU_DDEC')
    Y_TREE_AOCU_DANG = Widget_Tree(Y_TREENODE_AOCU_3, UNAME='Y_TREE_AOCU_DRANG'  $
      ,VALUE='d_Roll', UVALUE='AOCU_DRANG')

    Y_TREE_AOCU_CRS_OFF = Widget_Tree(Y_TREENODE_AOCU_3, UNAME='Y_TREE_AOCU_CRS_OFF'  $
      ,VALUE='Crs_Off', UVALUE='AOCU_CRS_OFF')

  ; (4) Quality
  Y_TREENODE_AOCU_4 = Widget_Tree(Y_TREENODE_AOCU, UNAME='Y_TREENODE_AOCU_4'  $
      ,/FOLDER ,VALUE='Quality')
    Y_TREE_AOCU_QUAL_INTP = Widget_Tree(Y_TREENODE_AOCU_4, UNAME='Y_TREE_AOCU_QUAL_INTP'  $
      ,VALUE='Interpolated', UVALUE='AOCU_QUAL_INTP')

  Widget_Control, parentID, /UPDATE

end