; $Id: mk_sub_tsd.pro,v 1.2 2006/03/23 06:23:01 baba Exp $
;+
; NAME:
;       MK_SUB_TSD
;
; PURPOSE:
;       The purpose of this procedure is to make a subset from
;       the current TSD. The TSD object which has been opend is
;       replaced by the new subset.
;       This procedure is called by FISv.
;
; AUTHOR:
;       ASTRO-F Data Reduction team
;       Web Page: http://www.ir.isas.jaxa.jp/ASTRO-F/iris_data/
;
; CATEGORY:
;       FIS Data Visualizer
;
; CALLING SEQUENCE:
;       mk_sub_tsd, GROUP_LEADER=Event.top
;
; ;
; MODIFICATION HISTORY:
;        Written by: Makiuti,S.
;        06/02/04 Last modified
;-

;-----------------------------------------------------------------
; Event CallBack Procedures
;-----------------------------------------------------------------
;
; Empty stub procedure used for autoloading.
;
pro mk_sub_tsd_eventcb
end

;-----------------------------------------------------------------
pro MakeBtn, Event
COMMON Share, tsd

  ; Get ID of DATA selector in slider0
  WID_SLIDER_0 = widget_info(Event.top, find_by_uname='WID_SLIDER_0')
  ; Get value selected in slider now
  widget_control, WID_SLIDER_0, get_value=slidevalue_start

  ; Get ID of DATA selector in slider2
  WID_SLIDER_2 = widget_info(Event.top, find_by_uname='WID_SLIDER_2')
  ; Get value selected in slider now
  widget_control, WID_SLIDER_2, get_value=slidevalue_end

  ;
  msg = 'Making subset TSD from ' + strcompress(string(slidevalue_start)) + $
         ' to ' + strcompress(string(slidevalue_end)) + ' frames.'
  print, msg

  ; Error check
  IF (slidevalue_start GT slidevalue_end) THEN BEGIN
    Result = DIALOG_MESSAGE("End frame number must be more than Start frame number",$
               DIALOG_PARENT=Event.top, /ERROR)
  ENDIF ELSE BEGIN

    ; Make a subset of the current TSD
    index = lindgen(slidevalue_end-slidevalue_start+1) + slidevalue_start
    subset = tsd->subset_TSD(index)
    OBJ_DESTROY, tsd
    tsd = subset

    WIDGET_CONTROL, Event.top, /DESTROY
  ENDELSE

end

;-----------------------------------------------------------------
pro CancelBtn, Event
  WIDGET_CONTROL, Event.top, /DESTROY
end

;-----------------------------------------------------------------
pro WID_BASE_SUBTSD_event, Event

  wTarget = (widget_info(Event.id,/NAME) eq 'TREE' ?  $
      widget_info(Event.id, /tree_root) : event.id)
  wWidget =  Event.top

  case wTarget of

    Widget_Info(wWidget, FIND_BY_UNAME='WID_BUTTON_0'): begin
      if( Tag_Names(Event, /STRUCTURE_NAME) eq 'WIDGET_BUTTON' )then $
        MakeBtn, Event
    end
    Widget_Info(wWidget, FIND_BY_UNAME='WID_BUTTON_1'): begin
      if( Tag_Names(Event, /STRUCTURE_NAME) eq 'WIDGET_BUTTON' )then $
        CancelBtn, Event
    end
    else:
  endcase

end

pro WID_BASE_SUBTSD, GROUP_LEADER=wGroup, _EXTRA=_VWBExtra_
COMMON Share, tsd

  device, get_screen_size=screen_size
  xoffset = screen_size[0]/4
  yoffset = screen_size[1]/3

  WID_BASE_SUBTSD = Widget_Base( GROUP_LEADER=wGroup, UNAME='WID_BASE_SUBTSD'  $
      ,XOFFSET=xoffset ,YOFFSET=yoffset ,TITLE='Make a subset from the current TSD' ,SPACE=3 ,XPAD=3  $
      ,YPAD=3 ,COLUMN=1)

  ;
  WID_BASE_1 = Widget_Base(WID_BASE_SUBTSD, UNAME='WID_BASE_1' ,FRAME=1  $
      ,XOFFSET=3 ,YOFFSET=3 ,SCR_XSIZE=512 ,SCR_YSIZE=65 ,TITLE='IDL'  $
      ,SPACE=3 ,XPAD=7 ,YPAD=3 ,COLUMN=1)

  WID_LABEL_0 = Widget_Label(WID_BASE_1, UNAME='WID_LABEL_0'  $
      ,XOFFSET=98 ,YOFFSET=3 ,/ALIGN_CENTER ,VALUE='Start of Frame')

  WID_SLIDER_0 = Widget_Slider(WID_BASE_1, UNAME='WID_SLIDER_0'  $
      ,XOFFSET=7 ,YOFFSET=20 ,SCR_XSIZE=250 ,SCR_YSIZE=33)

  ;
  WID_BASE_2 = Widget_Base(WID_BASE_SUBTSD, UNAME='WID_BASE_2' ,FRAME=1  $
      ,XOFFSET=3 ,YOFFSET=71 ,SCR_XSIZE=512 ,SCR_YSIZE=65  $
      ,TITLE='IDL' ,SPACE=3 ,XPAD=7 ,YPAD=3 ,COLUMN=1)

  WID_LABEL_2 = Widget_Label(WID_BASE_2, UNAME='WID_LABEL_2'  $
      ,XOFFSET=101 ,YOFFSET=3 ,/ALIGN_CENTER ,VALUE='End of Frame')

  WID_SLIDER_2 = Widget_Slider(WID_BASE_2, UNAME='WID_SLIDER_2'  $
      ,XOFFSET=7 ,YOFFSET=20 ,SCR_XSIZE=250 ,SCR_YSIZE=33)

  ;
  WID_BASE_3 = Widget_Base(WID_BASE_SUBTSD, UNAME='WID_BASE_3' ,XOFFSET=50  $
      ,YOFFSET=139 ,/ALIGN_CENTER ,TITLE='IDL' ,SPACE=5 ,XPAD=3  $
      ,YPAD=3 ,ROW=1 ,/GRID_LAYOUT)

  ; Make and Cancel Button
  Btns_Base = Widget_Base(WID_BASE_3, /row)
  WID_BUTTON_0 = Widget_Button(Btns_Base, UNAME='WID_BUTTON_0'  $
      ,XOFFSET=3 ,YOFFSET=3 ,SCR_XSIZE=105 ,SCR_YSIZE=23  $
      ,/ALIGN_CENTER ,VALUE='Make TSD Subset', Tooltip='Make TSD Subset')

  WID_BUTTON_1 = Widget_Button(Btns_Base, UNAME='WID_BUTTON_1'  $
      ,XOFFSET=68 ,YOFFSET=3 ,SCR_XSIZE=55 ,SCR_YSIZE=23  $
      ,/ALIGN_CENTER ,VALUE='Cancel', Tooltip='Cancel')


  ; Added code manually START
  ; Make TSD object for test
  ;tsd=Obj_New('tsd', INFILE='C:\RSI\IDL56\sample_fits\fis20020927_112819_03_lw.fits')

  ;## Set Values of Slidebars ##
  ; Make array of detector signals
  vdet=tsd->get_value_tsd('FIS_OBS', 'DET')
  ; Find number of array samples
  vdet_smp_max = n_elements(vdet[0,*])
;vdet_smp_max=100;;;;;;;;;;;;;;;;;

  ; Set slider max to the number of samples
  widget_control, WID_SLIDER_0, set_slider_max=vdet_smp_max-1
  widget_control, WID_SLIDER_2, set_slider_max=vdet_smp_max-1
  widget_control, WID_SLIDER_2, set_value=vdet_smp_max-1
  ; Added code manually END

  Widget_Control, /REALIZE, WID_BASE_SUBTSD

  XManager, 'WID_BASE_SUBTSD', WID_BASE_SUBTSD, /NO_BLOCK

end

;
; Empty stub procedure used for autoloading.
;
pro mk_sub_tsd, GROUP_LEADER=wGroup, _EXTRA=_VWBExtra_
  WID_BASE_SUBTSD, GROUP_LEADER=wGroup, _EXTRA=_VWBExtra_
end
