;#> frin_plot_result.dc3
; Identifier	frin_plot_result
;
; Purpose	plot the result of the fringe removal program
;
; Synopsis	frin_plot_result, fine, frid, ss, ww, yy, xx, nfrin, fill=fill
;
; Arguments	Name	I/O    Type:	Description
;		------------------------------------------------------------
;		fine	 I     struct	plot struct
;		frid	 I     struct	fringe identity
;		ss	 I     fltarr	smooth background
;		ww	 I     struct	wavelengths (micron)
;		yy	 I     struct	input data
;		xx	 I     struct	fringe data
;		nfrin	 I     struct	number of fringes found
;		fill	 I     struct	fill factor
;
; Returns
;
; Description
;
;
;
;
; Comment
;
;
; Example	--
;
; Category	UTIL ( module: fringes )
;
; Filename	frin_plot_result.pro
;
; Author	Do Kester
;
; Version	1.3
;
; History	1.0  08-03-2001 DK, first implementation of frin_plot_result.pro
;		1.1  05-04-2001 DK, remove compile_opt
;		1.2  16-05-2001 DK, remove minor tick marks in lower panel
;		1.3  19-02-2003 FL, cal25 only for SWS aot-bands
;#<
;===========================================================================

pro frin_plot_result, fine, frid, ss, ww, yy, xx, wgt, nfrin, fill=fill

compile_opt idl2, hidden

forward_function frin_title

winset, 2

fr = cliprange( fine.fr5 ) * 2
sr = cliprange( ss )
yr = [ 0.0, ( sr[1] + fr[1] ) * 1.2 ]
wr = cliprange( ww )
yo = ( yr[1] - yr[0] )
pltitle = frin_title( frid )

plot, wr, yr, xst=3, yst=3, pos=[0.1, 0.3, 0.9, 0.9], ytitle='flux',$
	xtickname=replicate( ' ', 10 ), title=pltitle, /nodata

plot_line, ww, yy, psym=3, /oplot
plot_line, ww, ss, psym=3, /oplot, color=2
plot_line, ww, ( xx + 1 ) * ss + 0.15 * yo, psym=3, /oplot, color=3

fr = ( yr - 0.5 * total( yr ) ) * 0.2 / 0.6
fine.wn5 = 1.0 / fine.wn5
frin_plot_part, ww, yy - ss, wgt, fine, yrange=fr, pos=[0.1, 0.1, 0.9, 0.3], $
		ps=3, symsize=1, yst=3, /noerase, xtitle='wavelength', $
		yminor=1, yticklen=0.01, ytitle='fringes'

if ( n_elements( fill ) GT 0 ) then oplot, fine.wn5, fill, color=2

;if ( frid.band EQ 0 ) then oplot, ww, yy - ss, ps=3
xyouts, 0.7, 0.26, string( nfrin, ' fringe components', format='(i2,a)' ), $
	/normal

return
end

