;#> plot_line.dc1
; Identifier	plot_line
;
; Purpose	plot a line in continuous segments
;
; Synopsis	plot_line, x, y, psym=psym, close=close, oplot=oplot, _extra=_e
;
; Arguments	Name	I/O    Type:	Description
;		------------------------------------------------------------
;		x	 I     fltarr	x coordinate of points to plot
;		y	 I     fltarr	y coordinate of points to plot
;		psym	 I     int(arr) plot symbols
;		close	 I     float	how close the points need to be
;					default : 6
;		oplot	 I     keyword	do overplotting
;		_extra	 I     struct	to be passedto theplotting routines
;
; Returns	--
;
; Description	Close adjacent points are connected by lines.
;		When points are more apart than CLOSE * median of the absolute
;		distance between the points, the connecting is stopped.
;
;		All points are plotted, either connected or as individual
;		points.
;
; Comment
;
;
; Example	--
;
; Category	UTIL ( module: plot_line )
;
; Filename	plot_line.pro
;
; Author	Do Kester
;
; Version	1.1
;
; History	1.0  21-03-2001 DK, first implementation of plot_line.pro
;		1.1  03-04-2001 DK, remove compile_opt
;		1.2  05-04-2001 DK, counting is difficult ...
;#<
;===========================================================================

pro plot_line, x, y, psym=psym, close=close, oplot=oplt, _extra=_e

compile_opt idl2

if ( n_elements( close ) EQ 0 ) then close = 6
if ( n_elements( psym  ) EQ 0 ) then psym = 3
oplot = keyword_set( oplt )

dx = abs( x - shift( x, 1 ) )
medx = close * median( dx )
ps = abs( psym )
bdx = [ ( where( dx GT medx, nbdx ) > 0 ), n_elements( x ) ]
if ( bdx[0] NE 0 ) then bdx = [ 0, bdx ]

if ( nbdx GT 0 ) then begin
  if ( oplot ) then oplot, [x[bdx]], [y[bdx]], ps=ps, _extra=_e $
  else plot, [x[bdx]], [y[bdx]], ps=ps, _extra=_e
  oplot = 1
endif

ps = -abs( psym )
for k = n_elements( bdx ) - 2, 0, -1 do begin
  b = lindgen( bdx[k+1] - bdx[k] ) + bdx[k]
  if ( oplot ) then oplot, [x[b]], [y[b]], ps=ps, _extra=_e $
  else plot, [x[b]], [y[b]], ps=ps, _extra=_e

  oplot = 1
endfor

return
end

;===========================================================================

