; $Id: extrac_data_by_ti.pro,v 1.2 2004/05/21 07:48:48 baba Exp $
;
; Copyright (c) 2003, ASTRO-F Data Reduction Team.  All rights reserved.
;
;+
; NAME:
;	extrac_data_by_ti.pro
; PURPOSE:
;	Data values specfied by CTIare extracted.
; CATEGORY:
;	Function. Libfary for the procedures in ASTRO-F pipeline.
; CALLING SEQUENCE:
;	Result = extrac_data_by_ti(tsd, start[, end]) 
; INPUTS:
;	tsd		TSD object reference.
;	start		Integer. Starting time obtained by FIS TI counter.
;	end		Integer. End CTI(optional.) To the end of file
;			if not specified.	
; OUTPUTS:
;	data 		Array of the detector data.
;	cnt		Number of resets within the TSD treated.
; RETURN:
;	data value	Value extracted
;	!FAIL in failure.
; RESTRICTIONS:
;	At this moment, only CTI is to be specified
; PROCEDURE:
;		
; EXAMPLE:
;	ex_vdet = extrac_data_by_ti(tsd, 2)
; REFERENCE:
;	
;
; MODIFICATION HISTORY:
; 	Mar. 2003		Written by A. Kawamura.
;-

FUNCTION extrac_data_by_ti, tsd, startsample, endsample
;ON_ERROR 2

;;; Check for position parameters ;;;
	IF (N_PARAMS(0) EQ 0) THEN BEGIN
	PRINT, 'Calling Sequence: result=extrac_data_by_time(tsd, start[, end])'
	PRINT, 'TSD = Object reference type of TSD, read by WRITE_TSD or OBJ_NEW.'
	PRINT, 'start = TI from which the data to be extracted'
	PRINT, 'end = End TI until which the data to be extracted'
	PRINT, 'OUTPUT;	Array of the detector data.'
	RETURN,!FAIL
	ENDIF

;;; Extract values from TSD object reference ;;;
	
	vdet = tsd->get_value_tsd('fis_obs','vdet')
	help, vdet
	si = SIZE(vdet)

;;; Check TI ;;;
	ti = tsd->get_value_tsd('fis_obs','aftime')
	IF (startsample GT ti[si(2)-1]) THEN BEGIN
	PRINT, 'This data starts from',ti[0],' and ends at',ti[si(2)-1]
	PRINT, 'NO Output...'
	RETURN,!FAIL
	ENDIF
	IF (ti[0] GT startsample) THEN BEGIN
	PRINT, 'This data starts from',ti[0]
	startsample=ti[0]
	ENDIF

	
	IF (N_PARAMS(0) EQ 2) THEN endsample =ti[si(2)-1] 		
	IF (ti[si(2)-1] LT endsample) OR (ti[0] GT endsample) THEN BEGIN
		PRINT,'This data ends at',ti[si(2)-1]
		endsample =ti[si(2)-1]
	ENDIF

	print,'from',startsample,' to',endsample
	ind = WHERE((ti GE startsample) and (ti LE endsample), count)
	print, count,' samples'
	help,ind

;;; extracting the data ;;;;

	ex_vdet=vdet[*,ind]

	RETURN, ex_vdet

END



