;
; Supplies function for cal_fit2_lw
;
; Define a return function for LMFIT:
;   f(x) = a[0]*(1-exp(-t/a[1])) + a[2]*(exp(-t/a[3])) + a[4]
Function cal2_lw_respons, t, a
  bt = exp(-t/a[1])
  dt = exp(-t/a[3])
  f  = a[0]*(1.D - bt)+a[2]*(dt)+a[4]

  Return, [f, 1.D - bt, -a[0]*bt*t/a[1]^2, dt, a[2]*dt*t/a[3]^2, 1.D]
End

;
; Performs curve fitting with the following function and returns y(t1)
; where t1 may be an array
;
;  ref. S.H.Oh et al. 2004
;
Function gb_cal_fit2_lw, y, t, t1, YFIT=YFit, PARAMS=Params, TSTART=t0, CHISQ=chisq, $
           ERR=err, FITA=FitA, ITMAX=itmax, ITMIN=itmin, ITER=iter

  ;  initial guess for the coefficients
  ;
  a = findgen(5)

  a[0] = max(y)*0.5
  a[2] = 1.4*max(y)
  a[4] = a[2]-y[0]*0.5

  a[1] = 100.
  a[3] = 2400.

  fita = [1,1,1,0,1]

  IF ~Keyword_Set(t0) THEN BEGIN
      t0  = t[0]
  ENDIF
  t_0 = t - t0
  function_name = 'cal2_lw_respons'
  c = lmfit(t_0, y, a, FITA=fita, FUNCTION_NAME=function_name, CHISQ=chisq, $
            /double, CONVERGENCE=convergence, ITMAX=300, MEASURE_ERROR=err)

  case convergence of
    1  : convergence=1 ; be quiet ;-)
    0  : LMESSAGE, 3, "% the algorithm '"+function_name+"' did not converge.",/CONTINUE
    -1 : LMESSAGE, 3, "% the algorithm '"+function_name+"' encountered a singular matrix and did not converge.",/CONTINUE
  endcase

  b = a[0]*(1.D - exp(-(t1-t0)/a[1])) + a[2]*exp(-(t1-t0)/a[3]) + a[4]

  IF arg_present(YFit)   NE 0 THEN YFit = c   ; Fitted result value at each point
  IF arg_present(Params) NE 0 THEN Params = a ; Parameters of the fitting function

  Return, b

End

