;
; Supplies function for cal_fit2_sw
;
; Define a return function for LMFIT:
;   f(x) = a[0]*(1-exp(-t/a[1])) + a[2]*(exp(-t/a[3])) + a[4]
Function cal2_sw_respons, t, a
  bt = exp(-t/a[1])
  dt = exp(-t/a[3])
  f  = a[0]*(1.D - bt)+a[2]*(dt)+a[4]

  Return, [f, 1.D - bt, -a[0]*bt*t/a[1]^2, dt, a[2]*dt*t/a[3]^2, 1.D]
End

Function cal2_sw_respons_jump, t, a
  bt = exp(-t/a[1])
  dt = exp(-t/a[3])
  f  = a[0]*bt+a[2]*(dt)+a[4]

  Return, [f, bt, a[0]*bt*t/a[1]^2, dt, a[2]*dt*t/a[3]^2, 1.D]
End


;
; Performs curve fitting with the following function and returns y(t1)
; where t1 may be an array
;
;  ref. S.H.Oh et al. 2004
;
Function gb_cal_fit2_sw, y, t, t1, YFIT=YFit, PARAMS=Params, TSTART=t0, CHISQ=chisq, FITA=FitA, ERR=err, ITMAX=itmax, ITMIN=itmin, ITER=iter

  ;  initial guess for the coefficients
  ;
  a = findgen(5)

  slop1 = (y[1]-y[0])/(t[1]-t[0])
;  slop2 = (y[2]-y[1])/(t[2]-t[1])
;  slop3 = (y[2]-y[0])/(t[2]-t[0])
  jump=0
  if slop1 LT 0 then jump=1
  if jump then begin
    function_name = 'cal2_sw_respons_jump'
    if keyword_set(Params) then begin
      a = Params
    endif else begin
      a[0] = y[0]*0.5
      a[2] = y[0]*0.7
      a[4] = y[0]*2
      a[1] = 50.
      a[3] = 2400.
    endelse
   if NOT keyword_set(FitA) then fita = [1,1,1,0,1]
;    a[0] = 0.00029811655  & a[1]=78.504301 & a[2]= 0.00010278123 &a[3]=2000.0000& a[4]=0.0012232674
;    fita = [1,1,1,1,1]
  endif else begin
    function_name = 'cal2_sw_respons'
    if keyword_set(Params) then begin
      a = Params
    endif else begin
      a[0] = max(y)*0.5
      a[2] = 1.4*max(y)
      a[4] = a[2]-y[0]*0.5
      a[1] = 200.
      a[3] = 2400.
    endelse
    if NOT keyword_set(FitA) then fita = [1,1,1,0,1]
  endelse
;  print,a

;  print, function_name
  IF ~Keyword_Set(t0) THEN BEGIN
      t0  = t[0]
  ENDIF
  t_0 = t - t0

  c = lmfit(t_0, y, a, FITA=fita, FUNCTION_NAME=function_name, CHISQ=chisq, $
            /double, CONVERGENCE=convergence, ITMAX=itmax, ITMIN=itmin, ITER=iter, MEASURE_ERROR=err)
;stop

  case convergence of
    1  : convergence=1 ; be quiet ;-)
    0  : LMESSAGE, 3, "% the algorithm '"+function_name+"' did not converge.",/CONTINUE
    -1 : LMESSAGE, 3, "% the algorithm '"+function_name+"' encountered a singular matrix and did not converge.",/CONTINUE
  endcase

;  print,fita
  if jump then begin
    b = a[0]*exp(-(t1-t0)/a[1]) + a[2]*exp(-(t1-t0)/a[3]) + a[4]
  endif else b = a[0]*(1.D - exp(-(t1-t0)/a[1])) + a[2]*exp(-(t1-t0)/a[3]) + a[4]

  IF arg_present(YFit)   NE 0 THEN YFit = c   ; Fitted result value at each point
  IF arg_present(Params) NE 0 THEN Params = a ; Parameters of the fitting function

  Return, b

End

