; $Id: gb_dark_fit.pro,v 1.5 2006/06/23 09:52:31 makiuti Exp $
;
; Copyright (c) 2002-2006, ASTRO-F Data Reduction Team.  All rights reserved.
;
;+
;
;FUNCTION NAME:
;       GB_Dark_Fit.pro
;
;CATEGORY:
;       Green Box Library Function in ASTRO-F pipeline.
;
;MODIFICATION HISTORY:
;                    Written by Y.S.Park & Kawamura
;       Jun. 2006    Last modified by Makiuti
;
;-

;;;  supplies function for dark_fit
Function dark_respons, t, a

  bt = exp(-t/a[1])
  dt = exp(-t/a[3])
  f  = a[0]*(1.D -bt)+a[2]*dt+a[4]

  Return, [f, 1.D -bt, -a[0]*bt*t/a[1]^2, dt, a[2]*dt*t/a[3]^2, 1.D]

End

;
; performs curve fitting with the following function and returns y(t1)
; where t1 may be an array
;
;  ref. S.H.Oh et al. 2004
;
Function gb_dark_fit, y, t, Err=err, CHISQ=chisq

;  initial guess for the coefficients
;
  a = findgen(5)
  a[0] = 0.D
  a[1] = 0.1D
  a[3] = 0.1D
  a[4] = min(y)
  a[2] = max(y)-a[4]

  fita = [0,0,1,1,1]

  t = t-t[0]

  c = lmfit(t,y,a, FITA=fita, FUNCTION_NAME='dark_respons', CHISQ=chisq, $
                SIGMA=sigma, /DOUBLE, CONVERGENCE=convergence, ITMAX=300)

  ;IF (a[4] le min(y)) THEN a[4]=min(y)
  IF (convergence NE 1) THEN BEGIN
     LMESSAGE, 3, 'GB_dark_fit did not converge.', /Continue
  ENDIF

  err = sigma[4]
  Return, a[4]

END
