;
; Copyright (c) 2002, ASTRO-F Data Reduction Team.  All rights reserved.
;
;+
; NAME:
;	mmt.pro
; PURPOSE:
;	Do MMT transform
; CATEGORY:
;	Green Box procedure in library in ASTRO-F pipeline.
; CALLING SEQUENCE:
;
; INPUTS:
;	data            input data
;       scale           scale for MMT 
; OUTPUTS:
;       transformed_data       
;
; RESTRICTIONS:
;	
; PROCEDURE:
;	1. This function will be provided by the European team.
; EXAMPLE:
;
; REFERENCE:
;
; MODIFICATION HISTORY:
; 	Mar. 2004		Written by T.Takagi
;    
;-

pro mmt, data, transformed_data, scale=scale

;; In the program, scale[0] is not used!!
if(n_elements(scale) eq 0) then scale=2L^[0,1,2,3,4,5,6,7] + 1
nscale = n_elements(scale)

transformed_data=fltarr(n_elements(data),n_elements(scale))
sum = fltarr(n_elements(data))
previous_filtered_data = data
for i=1,n_elements(scale)-1 do begin
    filtered_data=medsmooth(data,scale[i])
    transformed_data[*,i]=previous_filtered_data - filtered_data
    previous_filtered_data = filtered_data
    sum= temporary(sum)+reform(transformed_data[*,i])
endfor

;mmt_inverse, transformed_data, filtered_data, scale=scale

transformed_data[*,0]=data-sum

RETURN
END
