; $Id: tsd_fxaddhist.pro,v 1.4 2006/01/18 06:57:00 baba Exp $
;
; Copyright (c) 2003-2005, ASTRO-F Data Reduction Team.  All rights reserved.
;
;+
; NAME:
;   tsd_fxaddhis.pro
; PURPOSE:
;   Append the string to HISTORY in TSD header structure.
; CATEGORY:
;   Function. Libfary for the procedures in ASTRO-F pipeline.
;
; CALLING SEQUENCE:
;   Result = fxaddhist_tsd(tsd, newhstr)
;
; INPUTS:
;   tsd      Object reference type of TSD
;   newhstr  String to add HISTORY
;
; OUTPUTS:
;   tsd      Object reference type of TSD
;
; RETURN:
;   !SUCCEED in success.
;   !FAIL in failure.
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;   dummy=tsd_fxaddhist(tsd, newhstr)
;
; REFERENCE:
;
; MODIFICATION HISTORY:
;   May. 2003     Written by A. Kawamura.
;   Jul. 2005     Last modified by Makiuti
;-
FUNCTION tsd_fxaddhist, tsd, newhstr, curtime=curtime, prres=prres

;ON_ERROR 2

IF N_PARAMS(0) LT 2 OR N_PARAMS(0) GT 3 THEN BEGIN
    print, '  Calling Sequence: tsd_fxaddhist, tsd, newhstr (,/prres)'
    print, '       tsd = TSD object reference'
    print, '       newstr = new string to be added'
    print, '  when /curtime is set, current time is written in HISTORY'
    print, '  when /prres is set, new history is typed'
    return, !FAIL;
ENDIF

LMESSAGE, 7, 'Adding string to HISTORY', /CONTINUE

;;; Adding string to HISTORY ;;;
IF keyword_set(curtime) THEN BEGIN
   cur_time = SYSTIME(0)
   newhstr = cur_time + ':  ' + newhstr
ENDIF

dummy = tsd->Set_header_keyword_tsd('Primary','History', newhstr)

if keyword_set(prres) then BEGIN
   msg = STRCOMPRESS('"'+newhstr+'"')
   LMESSAGE, 5, msg, /CONTINUE
ENDIF
return, !SUCCEED

END

