;+
; NAME:
;   ss_Q_TO_RADEC.PRO
;
; PURPOSE:
;   calculate equatrial coodinate of each pixel from Q-parameters in GADS data
;
; CALLING SEQUENCE:
;
; MODIFICATION HISTORY:
;   Written by S. Matsuura besed on timage.pro by Suzuki, 2006/08/15
;   2006/10/03 Simplified by cutting non-used loops
;   2006/10/12 I.Yamamura renamed as ss_q_to_radec.pro
;   2007/02/02 I.Yamamura Alignment offset and aberration implemented.
;-
pro ss_q_to_radec, q, datatype, pix_pos_file, ra, dec

!p.multi = [0,0,0]
RAD2SEC = 206264.8062470963551572D ;[arcsec/rad]
RAD2DEG = 57.295779513082320877D   ;[deg/rad]
PI      = 3.1415926535897932385D

ABERRATION = 20.5D         ; only Earth's revolution

if( datatype eq 'FIS_SW' ) then begin
    print, datatype
    samp = 25.28D ;[Hz]
    DANN = 20L
    PIX  = 30D
    CH_NUM = 100L
    IN_SCAN_OFFSET    = -73D     ; from MLHES 07/02/02
    CROSS_SCAN_OFFSET = +25D     ; from MLHES 07/02/02
endif else begin
    print, datatype
    samp = 16.86D ;[Hz]
    DANN = 15L
    PIX  = 49D
    CH_NUM = 75L
    IN_SCAN_OFFSET    = -110D    ; from MLHES 07/02/02
    CROSS_SCAN_OFFSET = +30D     ; from MLHES 07/02/02
endelse

time_idx = n_elements(q)/4 ; Number of data in time series


; FIS pixel position $B%G!<%?$NFI9~(B  ########################################
openr,lun, pix_pos_file, /Get_LUN
   xpos = FLTARR(1000000)
   ypos = FLTARR(1000000)
   count = 0
   WHILE (NOT EOF(lun)) DO BEGIN
      READF, lun, ty, tx ; FPI$B:BI87O(B
      xpos[count] = -1D*tx ;  [arcsec]
      ypos[count] = 1D*ty ;  [arcsec]
      count = count + 1
   ENDWHILE
   xpos = xpos(0:count-1)
   ypos = ypos(0:count-1)
Free_lun,lun

rm0 = dblarr(3,3)
rm1 = dblarr(3,3)
rm2 = dblarr(3,3)
diag_sum = dblarr(4)
q2  = dblarr(4)
ra = dblarr(CH_NUM,time_idx)
dec = dblarr(CH_NUM,time_idx)

; boresight $B$N(BRA,DEC$B$X$N7W;;8!>ZMQ(B 
b1 = dblarr(time_idx)
b2 = dblarr(time_idx)
b3 = dblarr(time_idx)
ra_b = dblarr(time_idx)
dec_b = dblarr(time_idx)
for i=0L, time_idx-1 do begin
    b1[i] = 2*(q[0,i]*q[2,i] + q[1,i]*q[3,i]) ;
    b2[i] = 2*(q[1,i]*q[2,i] - q[0,i]*q[3,i]) ;
    b3[i] = - q[0,i]*q[0,i] - q[1,i]*q[1,i] + q[2,i]*q[2,i] + q[3,i]*q[3,i] ;
    
    if(b2[i] ge 0 ) then begin
        ra_b[i]=atan(b2[i],b1[i])*RAD2DEG ;   arctan(a2/a1)  
    endif else begin
        ra_b[i]=(2*PI+atan(b2[i],b1[i]))*RAD2DEG ;      
    endelse
    dec_b[i]=asin(b3[i])*RAD2DEG  ;
endfor

; estimate abberation and alignment correction 07/02/01
ra_bs = ra_b[time_idx/2]
dec_bs = dec_b[time_idx/2]
euler, ra_bs, dec_bs, lmd_bs, beta_bs, 3
print, ra_bs, dec_bs, lmd_bs, beta_bs

aberration = 5D + 20.5D * SIN(beta_bs / RAD2DEG)
xpos -= IN_SCAN_OFFSET + aberration
ypos += CROSS_SCAN_OFFSET


print,'Now calcurating RA. DEC. .....'
for ch = 0L, CH_NUM-1, 1L do begin

     ;boresight$B$+$i(Bpixel$B0LCV$X$N2sE>9TNs!J(Brm1$B!K$r:n@.(B (X$B<4$^$o$j"*(BY$B<4$^$o$j$N2sE>(B) ############ 
        rm1[0,0]= cos(xpos[ch]/RAD2SEC)       
        rm1[0,1]= 0.0D                    
        rm1[0,2]= sin(xpos[ch]/RAD2SEC)     
        rm1[1,0]= sin(ypos[ch]/RAD2SEC)*sin(xpos[ch]/RAD2SEC)
        rm1[1,1]= cos(ypos[ch]/RAD2SEC)     
        rm1[1,2]=-sin(ypos[ch]/RAD2SEC)*cos(xpos[ch]/RAD2SEC) 
        rm1[2,0]=-cos(ypos[ch]/RAD2SEC)*sin(xpos[ch]/RAD2SEC) 
        rm1[2,1]= sin(ypos[ch]/RAD2SEC)     
        rm1[2,2]= cos(xpos[ch]/RAD2SEC)*cos(ypos[ch]/RAD2SEC) 

    for i = 0L, time_idx-1, 1L do begin

      ;boresight $B$G$N(Bq-parameter $B$r2sE>9TNs!J(Brm0$B!K$KD>$9!#(B ##################################
      ;$B$3$N9TNs$O!"(BRA$B!&(BDEC$B:BI87O$+$i(BFPI$B:BI8$X$N(Battitude matrix
      ;matrix$BMWAG(B : $BNs$,(B1st idx, $B9T$,(B2nd idx $B$G$"$k$3$H$KCm0U!#(B
        rm0[0,0] = q[0,i]*q[0,i] - q[1,i]*q[1,i] - q[2,i]*q[2,i] + q[3,i]*q[3,i] ;
        rm0[1,0] = 2*(q[0,i]*q[1,i] + q[2,i]*q[3,i]) ;
        rm0[2,0] = 2*(q[0,i]*q[2,i] - q[1,i]*q[3,i]) ;
        rm0[0,1] = 2*(q[0,i]*q[1,i] - q[2,i]*q[3,i]) ;
        rm0[1,1] = - q[0,i]*q[0,i] + q[1,i]*q[1,i] - q[2,i]*q[2,i] + q[3,i]*q[3,i] ;
        rm0[2,1] = 2*(q[1,i]*q[2,i] + q[0,i]*q[3,i]) ;
        rm0[0,2] = 2*(q[0,i]*q[2,i] + q[1,i]*q[3,i]) ;
        rm0[1,2] = 2*(q[1,i]*q[2,i] - q[0,i]*q[3,i]) ;
        rm0[2,2] = - q[0,i]*q[0,i] - q[1,i]*q[1,i] + q[2,i]*q[2,i] + q[3,i]*q[3,i] ;


      ;RA$B!&(BDEC$B:BI87O$+$i3FAG;R$X$N2sE>9TNs(B(rm2)$B$r:n@.(B #######################################
      ;$B$H$"$kAG;R$N2sE>9TNs!JMWAG!K$N;~7ONs%G!<%?$r7W;;(B  rm2 = rm1 X rm0
        rm2 = rm1##rm0

        
      ;$B2sE>9TNs!J(Brm2$B!K$r(Bq-parameters$B$KJQ49$9$k(B ############################################
      ;  /*******************************************************
      ;  * there are four equivalent ways of calculating this.
      ;  * we pick the one with the greatest numerical accuracy
      ;  * for this matrix
      ;  ********************************************************/
        diag_sum[0] = 1 + rm2[0,0] - rm2[1,1] - rm2[2,2];
	diag_sum[1] = 1 - rm2[0,0] + rm2[1,1] - rm2[2,2];
	diag_sum[2] = 1 - rm2[0,0] - rm2[1,1] + rm2[2,2];
	diag_sum[3] = 1 + rm2[0,0] + rm2[1,1] + rm2[2,2];

	maxi = 0L;
	for j = 1L, 3L, 1L do begin
            if (diag_sum[j] gt diag_sum[maxi]) then begin
                maxi = j       
            endif
        endfor

	q2[maxi] = 0.5D * sqrt(diag_sum[maxi]);
	recip = 1D / (4D * q2[maxi]);

	if (maxi eq 0L) then begin
            q2[1] = recip * (rm2[1,0] + rm2[0,1]) ;
            q2[2] = recip * (rm2[0,2] + rm2[2,0]) ;
            q2[3] = recip * (rm2[2,1] - rm2[1,2]) ;
        endif else if(maxi eq 1L) then begin 
            q2[0] = recip * (rm2[1,0] + rm2[0,1]) ;
            q2[2] = recip * (rm2[2,1] + rm2[1,2]) ;
            q2[3] = recip * (rm2[0,2] - rm2[2,0]) ;
        endif else if(maxi eq 2L) then begin
            q2[0] = recip * (rm2[0,2] + rm2[2,0]) ;
            q2[1] = recip * (rm2[2,1] + rm2[1,2]) ;
            q2[3] = recip * (rm2[1,0] - rm2[0,1]) ;
        endif else if(maxi eq 3L) then begin
            q2[0] = recip * (rm2[2,1] - rm2[1,2]) ;
            q2[1] = recip * (rm2[0,2] - rm2[2,0]) ;
            q2[2] = recip * (rm2[1,0] - rm2[0,1]) ;
        endif


       ;$BF@$i$l$?(Bq-paramters$B!J(Bq2$B!K$+$i(BRA,DEC$B$KJQ49(B #########################################
        a1 = 2*(q2[0]*q2[2] + q2[1]*q2[3]) ;
        a2 = 2*(q2[1]*q2[2] - q2[0]*q2[3]) ;
        a3 = - q2[0]*q2[0] - q2[1]*q2[1] + q2[2]*q2[2] + q2[3]*q2[3] ;

        if(a2 ge 0 ) then begin
            ra[ch,i]=atan(a2,a1)*RAD2DEG ;   arctan(a2/a1)  
        endif else begin
            ra[ch,i]=(2*PI+atan(a2,a1))*RAD2DEG ;      
        endelse
            dec[ch,i]=asin(a3)*RAD2DEG ;
        endfor
endfor
help, ra,dec


END
