; $Id: se__define.pro,v 1.3 2006/03/23 06:23:22 baba Exp $
;
; Copyright(C) 2002, ASTRO-F Data Reduction Team.  All rights reserved.
;
;+
; CLASS_NAME:
;   SE - Binary Table Extension of FITS, inherited from BTE
;
; PURPOSE:
;   Define the structure for HDU, which is a part of TSD.
;
; CATEGORY:
;   Data definition
;
; CALLING SEQUENCE:
;   None
;
; INPUTS:
;   None
;
; OUTPUTS:
;   None
;
; RESTRICTIONS:
;   None
;
; COMMENTS:
;   None
;
; DESCRIPTION:
;   See 'ASTRO-F FIS/Survey Data Structure' in detail.
;
; EXAMPLE:
;   None
;
; SIDE EFFECTS:
;   None
;
; MODIFICATION HISTORY:
;   Written by: Hajime Baba, June 2002.
;
;-

;----------------------------------------------------------------------------
; Below is the definitions of SE specific functions.
;----------------------------------------------------------------------------

; FUNCTION se::Get_Status_Members
;     COMPILE_OPT hidden
;     IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL
;     CASE self.extver OF
;         4: members = [ 'BLANK', 'TYPE', 'DAYNIGHT', 'IN_SAA', 'IN_POLAR' ]
;         3: members = [ '', '', '', 'IN_POLAR', 'IN_SAA', 'DAYNIGHT', 'TYPE', 'BLANK' ]
;         2: members = [ '', '', '', 'IN_POLAR', 'IN_SAA', 'DAYNIGHT', 'TYPE', 'BLANK' ]
;         1: members = [ '', '', '', '', '', '', '', 'BLANK' ]
;         ELSE: MESSAGE, 'Unknown TSD version: ' + STRING(self.extver)
;     ENDCASE
;     RETURN, members
; END

; FUNCTION se::Get_Flag_Members
;     COMPILE_OPT hidden
;     IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL
;     CASE self.extver OF
;         4: members = [ '' ]
;         3: members = [ '' ]
;         2: members = [ '' ]
;         1: members = [ '' ]
;         ELSE: MESSAGE, 'Unknown TSD version: ' + STRING(self.extver)
;     ENDCASE
;     RETURN, members
; END

; FUNCTION se::Get_Pix_Flag_Members
;     COMPILE_OPT hidden
;     IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL
;     CASE self.extver OF
;         4: members = [ '' ]
;         3: members = [ '' ]
;         2: members = [ '' ]
;         1: members = [ '' ]
;         ELSE: MESSAGE, 'Unknown TSD version: ' + STRING(self.extver)
;     ENDCASE
;     RETURN, members
; END

; FUNCTION se::Get_Quality_Members
;     COMPILE_OPT hidden
;     IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL
;     CASE self.extver OF
;         4: members = [ {s,'',0} ]
;         3: members = [ {s,'',0} ]
;         2: members = [ {s,'',0} ]
;         1: members = [ {s,'',0} ]
;         ELSE: MESSAGE, 'Unknown TSD version: ' + STRING(self.extver)
;     ENDCASE
;     RETURN, members
; END


; INIT should return a scalar TRUE value if the initialization
; is successful, and FALSE if the initialization fails.
FUNCTION se::INIT, EXTVER=extver
    COMPILE_OPT hidden
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL
    RETURN, self->bte::INIT('SE', EXTVER=extver)
END

PRO se::CLEANUP
    COMPILE_OPT hidden
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL
    self->bte::CLEANUP
END

;----------------------------------------------------------------------------
;Routine to define the class structure
;
PRO se__define
    ; Not intended to be called directly by regular users
    COMPILE_OPT idl2, hidden
    ON_ERROR, 2                 ; Return to caller if an error occurs

    STRUCT_HIDE, { se, $   ; structure name
          INHERITS bte }        ; inheritance

END
