;----------------------------------------------------------------------------
;+
;
; NAME:
;   Write_TSDARR_To_LDS
;
; PURPOSE:
;   Write an array of TSD objects to LDS
;
; CALLING SEQUENCE:
;   result = Write_TSDARR_To_LDS(tsdarr, /KEEP)
;
; INPUTS:
;   NONE
;
; OPTIONAL KEYWORDS:
;   tsdarr	An array of TSD objects
;   /KEEP	Keep temporarily created file if set. (just for debug)
;
; OUTPUTS:
;   result	Result
;		Return !FAIL if fail.
;
; OUTPUT KEYWORDS:
;   NONE
;
; RESTRICTIONS:
;   Return values are undefined if both time and coordinates are specified.
;
; COMMENTS:
;   * Availability of the Internet connection.  In detail, be able to
;     access to the LDS server '!LDS_URL' via port 80 (HTTP).
;   * This method is for non-interactive use.  You can access to LDS
;     interactively via: !LDS_URL
;   * This function should not run in IDL demo mode.
; 
; EXAMPLE:
;     IDL> tsdarr = Get_TSDARR_From_LDS(MODE='lab', TIME='2004-07-10 23:40:00', SPAN=10, FORMAT='TSD3_LW')
;     IDL> (...)
;     IDL> result = Write_TSDARR_To_LDS(tsdarr)
;
;-
;----------------------------------------------------------------------------

FUNCTION Write_TSDARR_To_LDS, tsdarr, KEEP=keep
    COMPILE_OPT hidden
    ;;;;
    RESOLVE_ROUTINE, 'util'
    util
    setDebug
    ;;;;
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL

    IF NOT KEYWORD_SET(keep) THEN keep = 0

    num = N_ELEMENTS(tsdarr)
    IF num EQ 0 THEN BEGIN
        MESSAGE, 'No available data.', /CONTINUE
        RETURN, !FAIL
    ENDIF
    FOR i=0,num-1 DO BEGIN
        status = tsdarr[i]->Write_TSD_To_LDS()
        IF status EQ !FAIL THEN RETURN, !FAIL
    ENDFOR

    RETURN, !SUCCEED
END

